import sys
from PySide6 import QtWidgets, QtCore, QtGui
from PySide6.QtCore import QFile, QFileInfo, Qt, QByteArray, QTextStream, QStringDecoder
from PySide6.QtGui import QStandardItem, QStandardItemModel
from PySide6.QtWidgets import QApplication, QHeaderView, QTableView, QAbstractItemView

# datas = {
#     "Category 1": [
#         ("New Game 2", "Playnite", "", "", "Never", "Not Played", ""),
#         ("New Game 3", "Playnite", "", "", "Never", "Not Played", ""),
#     ],
#     "No Category": [
#         ("New Game", "Playnite", "", "", "Never", "Not Plated", ""),
#     ]
# }
#
#
# class GroupDelegate(QtWidgets.QStyledItemDelegate):
#     def __init__(self, parent=None):
#         super(GroupDelegate, self).__init__(parent)
#         self._plus_icon = QtGui.QIcon("../stuffs/add-fill.png")
#         self._minus_icon = QtGui.QIcon("../stuffs/subtract-fill.png")
#
#     def initStyleOption(self, option, index):
#         super(GroupDelegate, self).initStyleOption(option, index)
#         if not index.parent().isValid():
#             is_open = bool(option.state & QtWidgets.QStyle.State_Open)
#             option.features |= QtWidgets.QStyleOptionViewItem.HasDecoration
#             option.icon = self._minus_icon if is_open else self._plus_icon
#
#
# class GroupView(QtWidgets.QTreeView):
#     def __init__(self, model, parent=None):
#         super(GroupView, self).__init__(parent)
#         self.setIndentation(0)
#         self.setExpandsOnDoubleClick(False)
#         self.clicked.connect(self.on_clicked)
#         delegate = GroupDelegate(self)
#         self.setItemDelegateForColumn(0, delegate)
#         self.setModel(model)
#         self.header().setSectionResizeMode(0, QtWidgets.QHeaderView.ResizeToContents)
#         # self.header().setStyleSheet("background-color: #0D1225;")
#         self.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
#         self.setStyleSheet("background-color: #0D1225;")
#
#     @QtCore.Slot(QtCore.QModelIndex)
#     def on_clicked(self, index):
#         if not index.parent().isValid() and index.column() == 0:
#             self.setExpanded(index, not self.isExpanded(index))
#
#
# class GroupModel(QtGui.QStandardItemModel):
#     def __init__(self, parent=None):
#         super(GroupModel, self).__init__(parent)
#         self.setColumnCount(8)
#         self.setHorizontalHeaderLabels(["", "Name", "Library", "Release Date", "Genre(s)", "Last Played", "Time Played", ""])
#         for i in range(self.columnCount()):
#             it = self.horizontalHeaderItem(i)
#             it.setForeground(QtGui.QColor("#eb5959"))
#
#     def add_group(self, group_name):
#         item_root = QtGui.QStandardItem()
#         item_root.setEditable(False)
#         item = QtGui.QStandardItem(group_name)
#         item.setEditable(False)
#         ii = self.invisibleRootItem()
#         i = ii.rowCount()
#         for j, it in enumerate((item_root, item)):
#             ii.setChild(i, j, it)
#             ii.setEditable(False)
#         for j in range(self.columnCount()):
#             it = ii.child(i, j)
#             if it is None:
#                 it = QtGui.QStandardItem()
#                 ii.setChild(i, j, it)
#             it.setBackground(QtGui.QColor("#002842"))
#             it.setForeground(QtGui.QColor("#F2F2F2"))
#         return item_root
#
#     def append_element_to_group(self, group_item, texts):
#         j = group_item.rowCount()
#         item_icon = QtGui.QStandardItem()
#         item_icon.setEditable(False)
#         item_icon.setIcon(QtGui.QIcon("../stuffs/gamepad-fill.png"))
#         item_icon.setBackground(QtGui.QColor("#0D1225"))
#         group_item.setChild(j, 0, item_icon)
#         for i, text in enumerate(texts):
#             item = QtGui.QStandardItem(text)
#             item.setEditable(False)
#             item.setBackground(QtGui.QColor("#0D1225"))
#             item.setForeground(QtGui.QColor("#F2F2F2"))
#             group_item.setChild(j, i+1, item)
#
#
# class MainWindow(QtWidgets.QMainWindow):
#     def __init__(self, parent=None):
#         super(MainWindow, self).__init__(parent)
#
#         model = GroupModel(self)
#         tree_view = GroupView(model)
#         self.setCentralWidget(tree_view)
#
#         for group, childrens in datas.items():
#             group_item = model.add_group(group)
#             for children in childrens:
#                 model.append_element_to_group(group_item, children)
#
#
# if __name__ == '__main__':
#     app = QtWidgets.QApplication(sys.argv)
#     w = MainWindow()
#     w.resize(720, 240)
#     w.show()
#     sys.exit(app.exec())


class FreezeTableWidget(QTableView):
    def __init__(self, model):
        super(FreezeTableWidget, self).__init__()
        self.setModel(model)
        self.lastCol = self.model().columnCount() - 1
        self.frozenTableView = QTableView(self)
        self.init()
        self.horizontalHeader().sectionResized.connect(self.updateSectionWidth)
        self.verticalHeader().sectionResized.connect(self.updateSectionHeight)
        self.frozenTableView.verticalScrollBar().valueChanged.connect(
            self.verticalScrollBar().setValue)
        self.verticalScrollBar().valueChanged.connect(
            self.frozenTableView.verticalScrollBar().setValue)

    def init(self):
        self.frozenTableView.setModel(self.model())
        self.frozenTableView.setFocusPolicy(Qt.NoFocus)
        self.frozenTableView.verticalHeader().hide()
        self.frozenTableView.horizontalHeader().setSectionResizeMode(
                QHeaderView.Fixed)
        self.viewport().stackUnder(self.frozenTableView)

        self.frozenTableView.setStyleSheet('''
            QTableView { border: none;
                         background-color: #8EDE21;
                         selection-background-color: #999;
            }''')   # for demo purposes

        self.frozenTableView.setSelectionModel(self.selectionModel())
        # self.frozenTableView.setLayoutDirection(Qt.RightToLeft)
        for col in range(0, self.model().columnCount()):
            if col < self.lastCol:
                self.frozenTableView.setColumnHidden(col, True)
        self.frozenTableView.setColumnWidth(self.lastCol, self.columnWidth(self.lastCol))
        self.frozenTableView.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.frozenTableView.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.frozenTableView.show()
        self.updateFrozenTableGeometry()
        self.setHorizontalScrollMode(QAbstractItemView.ScrollPerPixel)
        self.setVerticalScrollMode(QAbstractItemView.ScrollPerPixel)
        self.frozenTableView.setVerticalScrollMode(QAbstractItemView.ScrollPerPixel)

    def updateSectionWidth(self, logicalIndex, oldSize, newSize):
        if logicalIndex == self.lastCol:
            self.frozenTableView.setColumnWidth(self.lastCol, newSize)
        self.updateFrozenTableGeometry()

    def updateSectionHeight(self, logicalIndex, oldSize, newSize):
        self.frozenTableView.setRowHeight(logicalIndex, newSize)

    def resizeEvent(self, event):
        super(FreezeTableWidget, self).resizeEvent(event)
        self.updateFrozenTableGeometry()

    def moveCursor(self, cursorAction, modifiers):
        current = super(FreezeTableWidget, self).moveCursor(cursorAction, modifiers)
        if (cursorAction == QAbstractItemView.MoveLeft and
                current.column() < self.lastCol and
                self.visualRect(current).topLeft().x() < self.frozenTableView.columnWidth(self.lastCol)):
            newValue = (self.horizontalScrollBar().value() +
                        self.visualRect(current).topLeft().x() -
                        self.frozenTableView.columnWidth(self.lastCol))
            self.horizontalScrollBar().setValue(newValue)
        return current

    def scrollTo(self, index, hint):
        if index.column() < self.lastCol:
            super(FreezeTableWidget, self).scrollTo(index, hint)

    def updateFrozenTableGeometry(self):
        x_position = self.verticalHeader().width() + self.frameWidth()
        for col in range(0, self.lastCol):
            x_position += self.columnWidth(col)
        x_viewPort = self.verticalHeader().width() + self.viewport().width() - self.columnWidth(self.lastCol) + self.frameWidth()
        self.frozenTableView.setGeometry(x_position if x_position < x_viewPort else x_viewPort,
                self.frameWidth(), self.columnWidth(self.lastCol),
                self.viewport().height() + self.horizontalHeader().height())


def main(args):
    # def split_and_strip(s, splitter):
    #     return [s.strip() for s in line.split(splitter)]

    app = QApplication(args)
    model = QStandardItemModel()
    data = [
        ["France", "Norway", "YDS", "UK(tech.)", "UK(adj.)", "UIAA", "Ger", "Australia", "Finland", "Brazil"],
        [1,0, 5.2,0, 0, "I", "I",0, 0, "Isup"],
        [2,0, 5.3,0,0 , "II", "II", 11,0, "II"],
        [3, 3, 5.4,0, 0, "III", "III", 12,0, "IIsup"],
        [4, 4, 5.5, "4a", "VD", "IV", "IV", 12,0, "III"],
        ["5a", 5 , 5.6,0, "S", "V" , "V", 13, 5 , "IIIsup"],
        ["5b", 5, 5.7, "4b", "HS", "V", "VI", 14, 5, "IV"],
        ["5c", 5 , 5.8,0, "VS", "VI", "VIIa", 16, 5, "IVsup"],
        [6, 6, 5.9, 5, "HVS", "VI", "VIIb", 17,0, "V"],
    ]
    header = data[0]
    model.setHorizontalHeaderLabels(header)
    row = 0
    for x in range(1, len(data)):
        fields = data[x]
        for col, field in enumerate(fields):
            newItem = QStandardItem(field)
            model.setItem(row, col, newItem)
        row += 1
    tableView = FreezeTableWidget(model)
    tableView.setWindowTitle("Frozen Column Example")
    tableView.resize(560, 680)
    tableView.show()
    return app.exec()


if __name__ == '__main__':
    main(sys.argv)










import uuid
from typing import Optional, Callable, Dict, Any

from dataclasses import dataclass, field
from functools import wraps
import uuid
from dataclasses import replace
from .use_state import useState



from ..immutable import Immutable

from redux import (
    BaseAction,
    BaseEvent,
    CompleteReducerResult,
    FinishAction,
    ReducerResult,
)
from redux.main import Store


class InitialState(Immutable):
  currentLocation: Optional[str] = ""
  params: Optional[dict] = field(default_factory=dict)



class UpdateCurrentLocationAction(BaseAction):
    currentLocation: Optional[str] = ""

class UpdatePushParamsAction(BaseAction):
    params: Optional[dict]
    

# reducer:
def reducer(
    state: InitialState | None,
    action: BaseAction,
) -> ReducerResult[InitialState, BaseAction, BaseEvent]:
    if state is None:
        return InitialState()
    

    if isinstance(action, UpdateCurrentLocationAction):
        return replace(
            state,
            currentLocation=action.currentLocation,
        )
    
    if isinstance(action, UpdatePushParamsAction):
        return replace(
            state,
            params=action.params,
        )
    
    return state


router_store = Store(reducer)
router_store.dispatch(UpdateCurrentLocationAction(currentLocation=""))
router_store.dispatch(UpdatePushParamsAction(params={}))

location, setLocation = useState(router_store._state.currentLocation)
params, setParams = useState(router_store._state.params)

def useRouter():
    global location
    class Route():
        def __init__(self):
            self.location = None
        def push(self, location):
            # router_store.dispatch(UpdateCurrentLocationAction(currentLocation=f"push//:{location}"))
            params = location.split('?')[1]
            params_list = params.split("&&")
            params_dict = {}
            for param in params_list:
                params_dict.update({param.split("=")[0]: param.split("=")[1]})
            router_store.dispatch(UpdatePushParamsAction(params=params_dict))
        def replace(self, location):
            router_store.dispatch(UpdateCurrentLocationAction(currentLocation=f"replace//:{location}"))
        def to(self, location):
            router_store.dispatch(UpdateCurrentLocationAction(currentLocation=location))
    route = Route()
    route.location = location
    return route



def useSearchParams():
    global params
    return params

def useLocation():
    global location, setLocation
    # setLocation(router_store._state.currentLocation)
    return location

@router_store.autorun(lambda state: state.currentLocation)
def update_location(currentLocation):
    setLocation(currentLocation)

@router_store.autorun(lambda state: state.params)
def update_params(params):
    setParams(params)





from typing import Dict, TYPE_CHECKING
if TYPE_CHECKING:
    from qtmui.material.styles.create_theme.theme_reducer import ThemeState

from .properties_name import *
from ....system.color_manipulator import alpha

def drawer(_theme) -> Dict:
    theme: ThemeState = _theme

    darkMode = theme.palette.mode == "dark"

    return {
        'PyDrawer': {
            'styles': {
                'root': lambda ownerState: {
                    # backgroundColor: f"rgba({0 if darkMode else 255}, {0 if darkMode else 255}, {0 if darkMode else 255}, 0.8)",
                    backgroundColor: theme.palette.background.paper,
                    # "background": "transparent",
                    # boxShadow: f"40px 40px 80px -8px {alpha(theme.palette.grey._500 if lightMode else theme.palette.common.black, 0.24)}" if ownerState.get("anchor") == "left" else f"-40px 40px 80px -8px {alpha(theme.palette.grey._500 if lightMode else theme.palette.common.black, 0.24)}"
                },
                'contentFrame': lambda ownerState: {
                    backgroundColor: theme.palette.background.paper,
                }
            }
        }
    }



class Alert:
    CLOSE = ':/icons/svg_cache/material-symbols_close-rounded.svg'
    ERROR = ':/icons/svg_cache/si_error-fill.svg'
    INFO = ':/icons/svg_cache/material-symbols_info-rounded.svg'
    SUCCESS = ':/icons/svg_cache/ep_success-filled.svg'
    WARNING = ':/icons/svg_cache/fluent_warning-28-filled.svg'




import uuid
from typing import Optional, Union, Dict, List, Callable
from PySide6.QtWidgets import QFrame, QVBoxLayout, QWidget, QLabel, QHBoxLayout, QSizePolicy
from PySide6.QtCore import Qt
from qtmui.hooks import State
from qtmui.material.styles import useTheme
from qtmui.material.widget_base.widget_base import PyWidgetBase
from ..typography import Typography
from ..utils.validate_params import _validate_param
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style

class ListItemText(QFrame, PyWidgetBase):
    """
    A component that renders primary and secondary text within a ListItem, styled like Material-UI ListItemText.

    The `ListItemText` component is used to display primary and optional secondary text in a ListItem, with support
    for typography customization, indentation, and styling overrides.

    Parameters
    ----------
    children : State, str, QWidget, List[Union[str, QWidget]], or None, optional
        Alias for the `primary` prop. The main content element. Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    disableTypography : State or bool, optional
        If True, the children won't be wrapped by a Typography component. Default is False.
        Can be a `State` object for dynamic updates.
    id : State or str, optional
        The identifier for the component. Default is None.
        Can be a `State` object for dynamic updates.
    inset : State or bool, optional
        If True, the children are indented. Default is False.
        Can be a `State` object for dynamic updates.
    onClick : State or Callable, optional
        Callback function triggered when the component is clicked. Default is None.
        Can be a `State` object for dynamic updates.
    primary : State, str, QWidget, List[Union[str, QWidget]], or None, optional
        The main content element. Default is None.
        Can be a `State` object for dynamic updates.
    primaryTypographyProps : State or dict, optional
        Props forwarded to the primary Typography component (if `disableTypography` is False). Default is None.
        Deprecated: Use `slotProps.primary` instead.
        Can be a `State` object for dynamic updates.
    secondary : State, str, QWidget, List[Union[str, QWidget]], or None, optional
        The secondary content element. Default is None.
        Can be a `State` object for dynamic updates.
    secondaryTypographyProps : State or dict, optional
        Props forwarded to the secondary Typography component (if `disableTypography` is False). Default is None.
        Deprecated: Use `slotProps.secondary` instead.
        Can be a `State` object for dynamic updates.
    slotProps : State or dict, optional
        Props for each slot (`primary`, `root`, `secondary`). Default is None.
        Can be a `State` object for dynamic updates.
    slots : State or dict, optional
        Components used for each slot (`primary`, `root`, `secondary`). Default is None.
        Can be a `State` object for dynamic updates.
    spacing : State or int, optional
        The spacing between primary and secondary content. Default is 6.
        Can be a `State` object for dynamic updates.
    sx : State, list, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QFrame` class,
        supporting props of the native component (e.g., parent, style, className).

    Notes
    -----
    - Props of the native component are supported via `**kwargs` (e.g., `parent`, `style`, `className`).
    - The `children` prop is an alias for `primary`. If both are provided, `primary` takes precedence.
    - The `primaryTypographyProps` and `secondaryTypographyProps` are deprecated; use `slotProps.primary` and `slotProps.secondary` instead.
    - The component uses a QVBoxLayout to stack primary and secondary content vertically.

    Demos:
    - ListItemText: https://qtmui.com/material-ui/qtmui-listitemtext/

    API Reference:
    - ListItemText API: https://qtmui.com/material-ui/api/list-item-text/
    """

    def __init__(
        self,
        children: Optional[Union[State, str, QWidget, List[Union[str, QWidget]]]] = None,
        classes: Optional[Union[State, Dict]] = None,
        disableTypography: Union[State, bool] = False,
        id: Optional[Union[State, str]] = None,
        inset: Union[State, bool] = False,
        onClick: Optional[Union[State, Callable]] = None,
        primary: Optional[Union[State, str, QWidget, List[Union[str, QWidget]]]] = None,
        primaryTypographyProps: Optional[Union[State, Dict]] = None,
        secondary: Optional[Union[State, str, QWidget, List[Union[str, QWidget]]]] = None,
        secondaryTypographyProps: Optional[Union[State, Dict]] = None,
        slotProps: Optional[Union[State, Dict[str, Union[Dict, Callable]]]] = None,
        slots: Optional[Union[State, Dict[str, str]]] = None,
        spacing: Union[State, int] = 6,
        sx: Optional[Union[State, List, Dict, Callable, str]] = None,
        **kwargs
    ):
        super().__init__(**kwargs)
        PyWidgetBase._setUpUi(self)
        
        self.setObjectName(f"ListItemText-{str(uuid.uuid4())}")

        self.theme = useTheme()
        self._widget_references = []

        # Set properties with validation
        self._set_children(children)
        self._set_classes(classes)
        self._set_disableTypography(disableTypography)
        self._set_id(id)
        self._set_inset(inset)
        self._set_onClick(onClick)
        self._set_primary(primary or children)  # primary takes precedence, else use children
        self._set_primaryTypographyProps(primaryTypographyProps)
        self._set_secondary(secondary)
        self._set_secondaryTypographyProps(secondaryTypographyProps)
        self._set_slotProps(slotProps)
        self._set_slots(slots)
        self._set_spacing(spacing)
        self._set_sx(sx)

        self._init_ui()


    # Setter and Getter methods
    @_validate_param(file_path="qtmui.material.listitemtext", param_name="children", supported_signatures=Union[State, str, QWidget, List, type(None)])
    def _set_children(self, value):
        """Assign value to children."""
        self._children = value

    def _get_children(self):
        """Get the children value."""
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.listitemtext", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        """Assign value to classes."""
        self._classes = value

    def _get_classes(self):
        """Get the classes value."""
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.listitemtext", param_name="disableTypography", supported_signatures=Union[State, bool])
    def _set_disableTypography(self, value):
        """Assign value to disableTypography."""
        self._disableTypography = value

    def _get_disableTypography(self):
        """Get the disableTypography value."""
        return self._disableTypography.value if isinstance(self._disableTypography, State) else self._disableTypography

    @_validate_param(file_path="qtmui.material.listitemtext", param_name="id", supported_signatures=Union[State, str, type(None)])
    def _set_id(self, value):
        """Assign value to id."""
        self._id = value

    def _get_id(self):
        """Get the id value."""
        return self._id.value if isinstance(self._id, State) else self._id

    @_validate_param(file_path="qtmui.material.listitemtext", param_name="inset", supported_signatures=Union[State, bool])
    def _set_inset(self, value):
        """Assign value to inset."""
        self._inset = value

    def _get_inset(self):
        """Get the inset value."""
        return self._inset.value if isinstance(self._inset, State) else self._inset

    @_validate_param(file_path="qtmui.material.listitemtext", param_name="onClick", supported_signatures=Union[State, Callable, type(None)])
    def _set_onClick(self, value):
        """Assign value to onClick."""
        self._onClick = value

    def _get_onClick(self):
        """Get the onClick value."""
        return self._onClick.value if isinstance(self._onClick, State) else self._onClick

    @_validate_param(file_path="qtmui.material.listitemtext", param_name="primary", supported_signatures=Union[State, str, Callable, QWidget, List, type(None)])
    def _set_primary(self, value):
        """Assign value to primary and store references."""
        self._primary = value

    def _get_primary(self):
        """Get the primary value."""
        return self._primary.value if isinstance(self._primary, State) else self._primary

    def _get_primary_widgets(self):
        """Get the widgets in primary."""
        primary = self._get_primary()
        if isinstance(primary, list):
            return [item for item in primary if isinstance(item, QWidget)]
        elif isinstance(primary, QWidget):
            return [primary]
        return []

    @_validate_param(file_path="qtmui.material.listitemtext", param_name="primaryTypographyProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_primaryTypographyProps(self, value):
        """Assign value to primaryTypographyProps."""
        self._primaryTypographyProps = value or {}

    def _get_primaryTypographyProps(self):
        """Get the primaryTypographyProps value."""
        return self._primaryTypographyProps.value if isinstance(self._primaryTypographyProps, State) else self._primaryTypographyProps

    @_validate_param(file_path="qtmui.material.listitemtext", param_name="secondary", supported_signatures=Union[State, str, QWidget, List, type(None)])
    def _set_secondary(self, value):
        """Assign value to secondary and store references."""
        self._widget_references = [w for w in self._widget_references if w not in self._get_secondary_widgets()]
        self._secondary = value
        secondary = value.value if isinstance(value, State) else value

        if isinstance(secondary, list):
            for item in secondary:
                if isinstance(item, QWidget):
                    self._widget_references.append(item)
                elif not isinstance(item, (str, type(None))):
                    raise TypeError(f"Each element in secondary must be a str, QWidget, or None, got {type(item)}")
        elif isinstance(secondary, QWidget):
            self._widget_references.append(secondary)
        elif secondary is not None and not isinstance(secondary, str):
            raise TypeError(f"secondary must be a State, str, QWidget, list, or None, got {type(secondary)}")

    def _get_secondary(self):
        """Get the secondary value."""
        return self._secondary.value if isinstance(self._secondary, State) else self._secondary

    def _get_secondary_widgets(self):
        """Get the widgets in secondary."""
        secondary = self._get_secondary()
        if isinstance(secondary, list):
            return [item for item in secondary if isinstance(item, QWidget)]
        elif isinstance(secondary, QWidget):
            return [secondary]
        return []

    @_validate_param(file_path="qtmui.material.listitemtext", param_name="secondaryTypographyProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_secondaryTypographyProps(self, value):
        """Assign value to secondaryTypographyProps."""
        self._secondaryTypographyProps = value or {}

    def _get_secondaryTypographyProps(self):
        """Get the secondaryTypographyProps value."""
        return self._secondaryTypographyProps.value if isinstance(self._secondaryTypographyProps, State) else self._secondaryTypographyProps

    @_validate_param(file_path="qtmui.material.listitemtext", param_name="slotProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_slotProps(self, value):
        """Assign value to slotProps."""
        self._slotProps = value or {}

    def _get_slotProps(self):
        """Get the slotProps value."""
        return self._slotProps.value if isinstance(self._slotProps, State) else self._slotProps

    @_validate_param(file_path="qtmui.material.listitemtext", param_name="slots", supported_signatures=Union[State, Dict, type(None)])
    def _set_slots(self, value):
        """Assign value to slots."""
        self._slots = value or {}

    def _get_slots(self):
        """Get the slots value."""
        return self._slots.value if isinstance(self._slots, State) else self._slots

    @_validate_param(file_path="qtmui.material.listitemtext", param_name="spacing", supported_signatures=Union[State, int])
    def _set_spacing(self, value):
        """Assign value to spacing."""
        self._spacing = value

    def _get_spacing(self):
        """Get the spacing value."""
        return self._spacing.value if isinstance(self._spacing, State) else self._spacing

    @_validate_param(file_path="qtmui.material.listitemtext", param_name="sx", supported_signatures=Union[State, List, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        """Assign value to sx."""
        self._sx = value

    def _get_sx(self):
        """Get the sx value."""
        return self._sx.value if isinstance(self._sx, State) else self._sx


    def _init_ui(self):

        # Tạo layout QVBoxLayout
        self._layout = QVBoxLayout(self)
        # self._layout.setContentsMargins(6, 0, 0, 0)
        self._layout.setSpacing(self._spacing)
        self.setLayout(self._layout)
        # self.layout().setAlignment(Qt.AlignCenter) # ra giữa theo chiều ngang và cả chiều dọc
        self.layout().setAlignment(Qt.AlignVCenter)  # căn giữa theo chiều dọc

        self.setAttribute(Qt.WA_TransparentForMouseEvents) # Điều này cho phép các sự kiện chuột (bao gồm hover) có thể đi qua và được lắng nghe bởi QPushButton.

        # Nếu có primary text, thêm nó vào layout
        if self._primary:
            primary_label = self._create_primary_label()
            self._layout.addWidget(primary_label)

        # Nếu có secondary text, thêm nó vào layout
        if self._secondary:
            secondary_label = self._create_secondary_label()
            self._layout.addWidget(secondary_label)

        self.slot_set_stylesheet()
        self.theme = useTheme()
        self.theme.state.valueChanged.connect(self.slot_set_stylesheet)

    def slot_set_stylesheet(self, value=None):
        self._set_stylesheet()

    def _set_stylesheet(self, component_styles=None):
        theme = useTheme()

        if not component_styles:
            component_styles = theme.components
        
        sx_qss = ""
        if self._sx:
            if isinstance(self._sx, dict):
                sx_qss = get_qss_style(self._sx)
            elif isinstance(self._sx, Callable):
                sx = self._sx()
                if isinstance(sx, dict):
                    sx_qss = get_qss_style(sx)
                elif isinstance(sx, str):
                    sx_qss = sx
            elif isinstance(self._sx, str) and self._sx != "":
                sx_qss = self._sx

        self.setStyleSheet(
            f"""
                ListItemText {{
                    {sx_qss}
                }}
            """
        )

    def _create_primary_label(self):
        """Tạo label cho primary text"""
        if self._disableTypography:
            # Không sử dụng Typography
            
            primary_label = QLabel(self._primary)
        else:
            if self._primaryTypographyProps:
                primary_label = QFrame()
                primary_label.setLayout(QHBoxLayout())
                primary_label.layout().setContentsMargins(0,0,0,0)
                primary_label.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
                if isinstance(self._primary, list):
                    for item in self._primary:
                        if isinstance(item, str):
                            text_item = Typography(text=item, **self._primaryTypographyProps)
                            primary_label.layout().addWidget(text_item)
                        elif isinstance(item, Callable):
                            text_item = Typography(text=item, **self._primaryTypographyProps)
                            primary_label.layout().addWidget(text_item)
                        elif isinstance(item, QWidget):
                            primary_label.layout().addWidget(item)
                else:
                    primary_label = Typography(text=self._primary, **self._primaryTypographyProps)
            else:
                primary_label = Typography(variant='subtitle2', text=self._primary, color='textPrimary')
        return primary_label

    def _create_secondary_label(self):
        """Tạo label cho secondary text"""
        # self.setMinimumHeight(48)

        if self._disableTypography:
            # Không sử dụng Typography
            secondary_label = QLabel(self._secondary)
        else:
            if self._secondaryTypographyProps:
                secondary_label = QFrame()
                secondary_label.setLayout(QHBoxLayout())
                secondary_label.layout().setContentsMargins(0,0,0,0)
                secondary_label.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
                if isinstance(self._secondary, list):
                    for item in self._secondary:
                        # print('item___________', item)
                        if isinstance(item, str):
                            text_item = Typography(text=item,**self._secondaryTypographyProps)
                            secondary_label.layout().addWidget(text_item)
                        elif isinstance(item, Callable):
                            text_item = Typography(text=item,**self._secondaryTypographyProps)
                            secondary_label.layout().addWidget(text_item)
                        elif isinstance(item, QWidget):
                            secondary_label.layout().addWidget(item)
                else:
                    secondary_label = Typography(text=self._secondary, **self._secondaryTypographyProps)
            else:
                secondary_label = Typography(variant='caption', text=self._secondary, color='textSecondary')
        return secondary_label

    def _apply_sx(self, sx):
        """Áp dụng overrides styles từ sx"""
        if isinstance(sx, dict):
            for key, value in sx.items():
                self.setStyleSheet(f"{key}: {value};")
        elif callable(sx):
            sx(self)

    def _apply_classes(self, classes):
        """Áp dụng các class CSS từ classes"""
        if isinstance(classes, dict):
            styles = "; ".join([f"{k}: {v}" for k, v in classes.items()])
            self.setStyleSheet(styles)

    def mouseReleaseEvent(self, event):
        if self._onClick:
            self._onClick()
        return super().mouseReleaseEvent(event)


import uuid
from typing import Optional, Union, Dict, List, Callable
from PySide6.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QSizePolicy, QFrame
from PySide6.QtCore import Signal, Qt
from PySide6.QtGui import QMouseEvent
from qtmui.hooks import State
from qtmui.material.styles import useTheme
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from ..widget_base import PyWidgetBase
from ..typography import Typography
from ..utils.validate_params import _validate_param

class Paper(QFrame, PyWidgetBase):
    """
    A component that renders a paper-like surface, styled like Material-UI Paper.

    The `Paper` component provides a container with elevation, rounded corners, and customizable styles,
    suitable for cards, dialogs, or other surfaces. It integrates with `Typography` and other components
    in the `qtmui` framework, retaining all existing parameters and aligning with MUI props.

    Parameters
    ----------
    children : State, str, QWidget, List[QWidget], or None, optional
        The content of the component. Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    component : State, type, str, or None, optional
        The component used for the root node (e.g., QFrame). Default is None (uses QFrame).
        Can be a `State` object for dynamic updates.
    direction : State or str, optional
        Layout direction ("column" or "row"). Default is "column".
        Can be a `State` object for dynamic updates.
    elevation : State or int, optional
        Shadow depth (0-24). Default is 1.
        Can be a `State` object for dynamic updates.
    key : State or str, optional
        Identifier for the component. Default is None.
        Can be a `State` object for dynamic updates.
    square : State or bool, optional
        If True, rounded corners are disabled. Default is False.
        Can be a `State` object for dynamic updates.
    sx : State, list, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    variant : State or str, optional
        Variant of the component ("elevation" or "outlined"). Default is "elevation".
        Can be a `State` object for dynamic updates.
    fullWidth : State or bool, optional
        If True, the component takes full width. Default is True.
        Can be a `State` object for dynamic updates.
    spacing : State or int, optional
        Spacing between children (in theme.spacing units). Default is 6.
        Can be a `State` object for dynamic updates.
    onClick : State or Callable, optional
        Callback fired when the component is clicked. Default is None.
        Can be a `State` object for dynamic updates.
    width : State or int, optional
        Fixed width in pixels. Default is None.
        Can be a `State` object for dynamic updates.
    height : State or int, optional
        Fixed height in pixels. Default is None.
        Can be a `State` object for dynamic updates.
    borderRadius : State or int, optional
        Border radius in pixels. Default is None (uses theme.shape.borderRadius).
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QFrame` class,
        supporting props of the native component (e.g., id, className).

    Signals
    -------
    themeChanged : Signal
        Emitted when the theme changes.
    clicked : Signal
        Emitted when the component is clicked.

    Notes
    -----
    - All existing parameters from the previous implementation are retained.
    - Props of the native component are supported via `**kwargs`.
    - The `elevation` prop uses MUI's box-shadow spec (0-24 levels).
    - MUI classes applied: `MuiPaper-root`, `MuiPaper-elevation`, `MuiPaper-outlined`.
    - Integrates with `Typography` for string children and other `qtmui` components.

    Demos:
    - Paper: https://qtmui.com/material-ui/qtmui-paper/

    API Reference:
    - Paper API: https://qtmui.com/material-ui/api/paper/
    """

    themeChanged = Signal()
    clicked = Signal()

    VALID_DIRECTIONS = ["column", "row"]
    VALID_VARIANTS = ["elevation", "outlined"]

    # MUI box-shadow spec (simplified for Qt stylesheet)
    SHADOWS = {
        0: "none",
        1: "0px 2px 1px -1px rgba(0,0,0,0.2), 0px 1px 1px 0px rgba(0,0,0,0.14), 0px 1px 3px 0px rgba(0,0,0,0.12)",
        2: "0px 3px 1px -2px rgba(0,0,0,0.2), 0px 2px 2px 0px rgba(0,0,0,0.14), 0px 1px 5px 0px rgba(0,0,0,0.12)",
        3: "0px 3px 3px -2px rgba(0,0,0,0.2), 0px 3px 4px 0px rgba(0,0,0,0.14), 0px 1px 8px 0px rgba(0,0,0,0.12)",
        4: "0px 2px 4px -1px rgba(0,0,0,0.2), 0px 4px 5px 0px rgba(0,0,0,0.14), 0px 1px 10px 0px rgba(0,0,0,0.12)",
        # ... (up to 24, simplified here for brevity)
        24: "0px 11px 15px -7px rgba(0,0,0,0.2), 0px 24px 38px 3px rgba(0,0,0,0.14), 0px 9px 46px 8px rgba(0,0,0,0.12)"
    }

    def __init__(
        self,
        children: Optional[Union[State, str, QWidget, List[QWidget]]] = None,
        classes: Optional[Union[State, Dict]] = None,
        component: Optional[Union[State, type, str]] = None,
        direction: Union[State, str] = "column",
        elevation: Union[State, int] = 1,
        key: Optional[Union[State, str]] = None,
        square: Union[State, bool] = False,
        sx: Optional[Union[State, List, Dict, Callable, str]] = None,
        variant: Union[State, str] = "elevation",
        fullWidth: Union[State, bool] = True,
        spacing: Union[State, int] = 6,
        onClick: Optional[Union[State, Callable]] = None,
        width: Optional[Union[State, int]] = None,
        height: Optional[Union[State, int]] = None,
        borderRadius: Optional[Union[State, int]] = None,
        *args,
        **kwargs
    ):
        super().__init__(*args, **kwargs)
        self.setObjectName(f"Paper-{str(uuid.uuid4())}")
        PyWidgetBase._setUpUi(self)

        self.theme = useTheme()
        self._widget_references = []

        # Set properties with validation
        self._set_children(children)
        self._set_classes(classes)
        self._set_component(component)
        self._set_direction(direction)
        self._set_elevation(elevation)
        self._set_key(key)
        self._set_square(square)
        self._set_sx(sx)
        self._set_variant(variant)
        self._set_fullWidth(fullWidth)
        self._set_spacing(spacing)
        self._set_onClick(onClick)
        self._set_width(width)
        self._set_height(height)
        self._set_borderRadius(borderRadius)

        self._init_ui()
        self._set_stylesheet()

        self.useEffect(
            self._set_stylesheet,
            [theme.state]
        )
        self.destroyed.connect(self._on_destroyed)
        self._connect_signals()

    # Setter and Getter methods
    @_validate_param(
        file_path="qtmui.material.paper",
        param_name="children",
        supported_signatures=Union[State, str, QWidget, List, type(None)]
    )
    def _set_children(self, value):
        """Assign value to children and store references."""
        self._widget_references.clear()
        self._children = value
        children = value.value if isinstance(value, State) else value

        if isinstance(children, list):
            for child in children:
                if not isinstance(child, QWidget):
                    raise TypeError(f"Each element in children must be a QWidget, got {type(child)}")
                self._widget_references.append(child)
        elif isinstance(children, (QWidget, str)):
            if isinstance(children, str):
                typography = Typography(text=children, variant="body1")
                self._widget_references.append(typography)
            else:
                self._widget_references.append(children)
        elif children is not None:
            raise TypeError(f"children must be a State, str, QWidget, List[QWidget], or None, got {type(children)}")

    def _get_children(self):
        """Get the children value."""
        children = self._children.value if isinstance(self._children, State) else self._children
        if isinstance(children, str):
            return [Typography(text=children, variant="body1")]
        return children if isinstance(children, list) else [children] if isinstance(children, QWidget) else []

    @_validate_param(
        file_path="qtmui.material.paper",
        param_name="classes",
        supported_signatures=Union[State, Dict, type(None)]
    )
    def _set_classes(self, value):
        """Assign value to classes."""
        self._classes = value

    def _get_classes(self):
        """Get the classes value."""
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(
        file_path="qtmui.material.paper",
        param_name="component",
        supported_signatures=Union[State, type, str, type(None)]
    )
    def _set_component(self, value):
        """Assign value to component."""
        self._component = value

    def _get_component(self):
        """Get the component value."""
        return self._component.value if isinstance(self._component, State) else self._component or QFrame

    @_validate_param(
        file_path="qtmui.material.paper",
        param_name="direction",
        supported_signatures=Union[State, str],
        valid_values=VALID_DIRECTIONS
    )
    def _set_direction(self, value):
        """Assign value to direction."""
        self._direction = value

    def _get_direction(self):
        """Get the direction value."""
        return self._direction.value if isinstance(self._direction, State) else self._direction

    @_validate_param(
        file_path="qtmui.material.paper",
        param_name="elevation",
        supported_signatures=Union[State, int],
        validator=lambda x: 0 <= x <= 24 if isinstance(x, int) else True
    )
    def _set_elevation(self, value):
        """Assign value to elevation."""
        self._elevation = value

    def _get_elevation(self):
        """Get the elevation value."""
        return self._elevation.value if isinstance(self._elevation, State) else self._elevation

    @_validate_param(
        file_path="qtmui.material.paper",
        param_name="key",
        supported_signatures=Union[State, str, type(None)]
    )
    def _set_key(self, value):
        """Assign value to key."""
        self._key = value

    def _get_key(self):
        """Get the key value."""
        return self._key.value if isinstance(self._key, State) else self._key

    @_validate_param(
        file_path="qtmui.material.paper",
        param_name="square",
        supported_signatures=Union[State, bool]
    )
    def _set_square(self, value):
        """Assign value to square."""
        self._square = value

    def _get_square(self):
        """Get the square value."""
        return self._square.value if isinstance(self._square, State) else self._square

    @_validate_param(
        file_path="qtmui.material.paper",
        param_name="sx",
        supported_signatures=Union[State, List, Dict, Callable, str, type(None)]
    )
    def _set_sx(self, value):
        """Assign value to sx."""
        self._sx = value

    def _get_sx(self):
        """Get the sx value."""
        return self._sx.value if isinstance(self._sx, State) else self._sx

    @_validate_param(
        file_path="qtmui.material.paper",
        param_name="variant",
        supported_signatures=Union[State, str],
        valid_values=VALID_VARIANTS
    )
    def _set_variant(self, value):
        """Assign value to variant."""
        self._variant = value
        self.setProperty("variant", self._get_variant())

    def _get_variant(self):
        """Get the variant value."""
        return self._variant.value if isinstance(self._variant, State) else self._variant

    @_validate_param(
        file_path="qtmui.material.paper",
        param_name="fullWidth",
        supported_signatures=Union[State, bool]
    )
    def _set_fullWidth(self, value):
        """Assign value to fullWidth."""
        self._fullWidth = value

    def _get_fullWidth(self):
        """Get the fullWidth value."""
        return self._fullWidth.value if isinstance(self._fullWidth, State) else self._fullWidth

    @_validate_param(
        file_path="qtmui.material.paper",
        param_name="spacing",
        supported_signatures=Union[State, int],
        validator=lambda x: x >= 0 if isinstance(x, int) else True
    )
    def _set_spacing(self, value):
        """Assign value to spacing."""
        self._spacing = value

    def _get_spacing(self):
        """Get the spacing value."""
        return self._spacing.value if isinstance(self._spacing, State) else self._spacing

    @_validate_param(
        file_path="qtmui.material.paper",
        param_name="onClick",
        supported_signatures=Union[State, Callable, type(None)]
    )
    def _set_onClick(self, value):
        """Assign value to onClick."""
        self._onClick = value

    def _get_onClick(self):
        """Get the onClick value."""
        return self._onClick.value if isinstance(self._onClick, State) else self._onClick

    @_validate_param(
        file_path="qtmui.material.paper",
        param_name="width",
        supported_signatures=Union[State, int, type(None)],
        validator=lambda x: x > 0 if isinstance(x, int) else True
    )
    def _set_width(self, value):
        """Assign value to width."""
        self._width = value

    def _get_width(self):
        """Get the width value."""
        return self._width.value if isinstance(self._width, State) else self._width

    @_validate_param(
        file_path="qtmui.material.paper",
        param_name="height",
        supported_signatures=Union[State, int, type(None)],
        validator=lambda x: x > 0 if isinstance(x, int) else True
    )
    def _set_height(self, value):
        """Assign value to height."""
        self._height = value

    def _get_height(self):
        """Get the height value."""
        return self._height.value if isinstance(self._height, State) else self._height

    @_validate_param(
        file_path="qtmui.material.paper",
        param_name="borderRadius",
        supported_signatures=Union[State, int, type(None)],
        validator=lambda x: x >= 0 if isinstance(x, int) else True
    )
    def _set_borderRadius(self, value):
        """Assign value to borderRadius."""
        self._borderRadius = value

    def _get_borderRadius(self):
        """Get the borderRadius value."""
        return self._borderRadius.value if isinstance(self._borderRadius, State) else self._borderRadius

    def _init_ui(self):
        """Initialize the UI based on props."""
        component = self._get_component()
        if not isinstance(self, component):
            # Re-instantiate with the correct component class
            self.__class__ = type("DynamicPaper", (component, PyWidgetBase), {})
        PyWidgetBase._setUpUi(self)

        # Set layout
        self.setLayout(QVBoxLayout() if self._get_direction() == "column" else QHBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)
        # self.layout().setSpacing(self.theme.spacing(self._get_spacing()))

        # Apply size policy
        width_policy = QSizePolicy.Expanding if self._get_fullWidth() else QSizePolicy.Preferred
        self.setSizePolicy(width_policy, QSizePolicy.Preferred)

        # Apply fixed dimensions
        if self._get_width():
            self.setFixedWidth(self._get_width())
        if self._get_height():
            self.setFixedHeight(self._get_height())

        # Clear previous widgets
        while self.layout().count():
            item = self.layout().takeAt(0)
            if item.widget():
                item.widget().setParent(None)

        # Apply children
        children = self._get_children()
        for child in children:
            self.layout().addWidget(child)

    def _set_stylesheet(self, component_styled=None):
        """Set the stylesheet for the Paper."""
        self.theme = useTheme()
        component_styled = component_styled or self.theme.components
        paper_styles = component_styled.get("Paper", {}).get("styles", {})
        root_styles = paper_styles.get("root", {})
        outlined_styles = paper_styles.get("outlined", {})
        root_qss = get_qss_style(root_styles)
        outlined_qss = get_qss_style(outlined_styles)

        # Handle sx
        sx = self._get_sx()
        sx_qss = ""
        if sx:
            if isinstance(sx, (list, dict)):
                sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
            elif isinstance(sx, Callable):
                sx_result = sx()
                if isinstance(sx_result, (list, dict)):
                    sx_qss = get_qss_style(sx_result, class_name=f"#{self.objectName()}")
                elif isinstance(sx_result, str):
                    sx_qss = sx_result
            elif isinstance(sx, str) and sx != "":
                sx_qss = sx

        # Handle classes
        classes = self._get_classes()
        classes_qss = get_qss_style(classes, class_name=f"#{self.objectName()}") if classes else ""

        # Apply MUI classes
        mui_classes = ["MuiPaper-root"]
        if self._get_variant() == "elevation":
            mui_classes.append(f"MuiPaper-elevation{self._get_elevation()}")
        else:
            mui_classes.append("MuiPaper-outlined")

        # Apply elevation
        elevation = min(max(self._get_elevation(), 0), 24)
        shadow = self.SHADOWS.get(elevation, self.SHADOWS[1])

        # Apply border radius
        border_radius = 0 if self._get_square() else (self._get_borderRadius() or self.theme.shape.borderRadius)
        border_radius_qss = f"border-radius: {border_radius}px;"

        # Apply variant
        variant_qss = f"border: 1px solid {self.theme.palette.divider};" if self._get_variant() == "outlined" else ""

        # stylesheet = f"""
        #     #{self.objectName()} {{
        #         {root_qss}
        #         {classes_qss}
        #         background-color: {self.theme.palette.background.paper};
        #         box-shadow: {shadow};
        #         {border_radius_qss}
        #         {variant_qss}
        #     }}
        #     #{self.objectName()}[variant=outlined] {{
        #         {outlined_qss}
        #     }}
        #     {sx_qss}
        # """
        stylesheet = f"""
            #{self.objectName()} {{
                {root_qss}
                {classes_qss}
                background-color: {self.theme.palette.background.paper};
                {border_radius_qss}
                {variant_qss}
            }}
            #{self.objectName()}[variant=outlined] {{
                {outlined_qss}
            }}
            {sx_qss}
        """
        self.setStyleSheet(stylesheet)

    def mouseReleaseEvent(self, event: QMouseEvent):
        """Handle mouse release event."""
        if event.button() == Qt.LeftButton and self._get_onClick():
            self._get_onClick()(event)
            self.clicked.emit()
        super().mouseReleaseEvent(event)

    def paintEvent(self, event):
        """Handle paint event."""
        PyWidgetBase.paintEvent(self, event)
        super().paintEvent(event)

    def resizeEvent(self, event):
        """Handle resize event."""
        PyWidgetBase.resizeEvent(self, event)
        super().resizeEvent(event)

    def _connect_signals(self):
        """Connect valueChanged signals of State parameters to their slots."""
        if isinstance(self._children, State):
            self._children.valueChanged.connect(self._on_children_changed)
        if isinstance(self._classes, State):
            self._classes.valueChanged.connect(self._on_classes_changed)
        if isinstance(self._component, State):
            self._component.valueChanged.connect(self._on_component_changed)
        if isinstance(self._direction, State):
            self._direction.valueChanged.connect(self._on_direction_changed)
        if isinstance(self._elevation, State):
            self._elevation.valueChanged.connect(self._on_elevation_changed)
        if isinstance(self._key, State):
            self._key.valueChanged.connect(self._on_key_changed)
        if isinstance(self._square, State):
            self._square.valueChanged.connect(self._on_square_changed)
        if isinstance(self._sx, State):
            self._sx.valueChanged.connect(self._on_sx_changed)
        if isinstance(self._variant, State):
            self._variant.valueChanged.connect(self._on_variant_changed)
        if isinstance(self._fullWidth, State):
            self._fullWidth.valueChanged.connect(self._on_fullWidth_changed)
        if isinstance(self._spacing, State):
            self._spacing.valueChanged.connect(self._on_spacing_changed)
        if isinstance(self._onClick, State):
            self._onClick.valueChanged.connect(self._on_onClick_changed)
        if isinstance(self._width, State):
            self._width.valueChanged.connect(self._on_width_changed)
        if isinstance(self._height, State):
            self._height.valueChanged.connect(self._on_height_changed)
        if isinstance(self._borderRadius, State):
            self._borderRadius.valueChanged.connect(self._on_borderRadius_changed)

    def _on_children_changed(self):
        """Handle changes to children."""
        self._set_children(self._children)
        self._init_ui()

    def _on_classes_changed(self):
        """Handle changes to classes."""
        self._set_classes(self._classes)
        self._set_stylesheet()

    def _on_component_changed(self):
        """Handle changes to component."""
        self._set_component(self._component)
        self._init_ui()

    def _on_direction_changed(self):
        """Handle changes to direction."""
        self._set_direction(self._direction)
        self._init_ui()

    def _on_elevation_changed(self):
        """Handle changes to elevation."""
        self._set_elevation(self._elevation)
        self._set_stylesheet()

    def _on_key_changed(self):
        """Handle changes to key."""
        self._set_key(self._key)

    def _on_square_changed(self):
        """Handle changes to square."""
        self._set_square(self._square)
        self._set_stylesheet()

    def _on_sx_changed(self):
        """Handle changes to sx."""
        self._set_sx(self._sx)
        self._set_stylesheet()

    def _on_variant_changed(self):
        """Handle changes to variant."""
        self._set_variant(self._variant)
        self._set_stylesheet()

    def _on_fullWidth_changed(self):
        """Handle changes to fullWidth."""
        self._set_fullWidth(self._fullWidth)
        self._init_ui()

    def _on_spacing_changed(self):
        """Handle changes to spacing."""
        self._set_spacing(self._spacing)
        self._init_ui()

    def _on_onClick_changed(self):
        """Handle changes to onClick."""
        self._set_onClick(self._onClick)

    def _on_width_changed(self):
        """Handle changes to width."""
        self._set_width(self._width)
        self._init_ui()

    def _on_height_changed(self):
        """Handle changes to height."""
        self._set_height(self._height)
        self._init_ui()

    def _on_borderRadius_changed(self):
        """Handle changes to borderRadius."""
        self._set_borderRadius(self._borderRadius)
        self._set_stylesheet()

    def _on_destroyed(self):
        """Clean up connections when the widget is destroyed."""
        if hasattr(self, "theme"):
            self.theme.state.valueChanged.disconnect(self._set_stylesheet)
        if isinstance(self._children, State):
            self._children.valueChanged.disconnect(self._on_children_changed)
        if isinstance(self._classes, State):
            self._classes.valueChanged.disconnect(self._on_classes_changed)
        if isinstance(self._component, State):
            self._component.valueChanged.disconnect(self._on_component_changed)
        if isinstance(self._direction, State):
            self._direction.valueChanged.disconnect(self._on_direction_changed)
        if isinstance(self._elevation, State):
            self._elevation.valueChanged.disconnect(self._on_elevation_changed)
        if isinstance(self._key, State):
            self._key.valueChanged.disconnect(self._on_key_changed)
        if isinstance(self._square, State):
            self._square.valueChanged.disconnect(self._on_square_changed)
        if isinstance(self._sx, State):
            self._sx.valueChanged.disconnect(self._on_sx_changed)
        if isinstance(self._variant, State):
            self._variant.valueChanged.disconnect(self._on_variant_changed)
        if isinstance(self._fullWidth, State):
            self._fullWidth.valueChanged.disconnect(self._on_fullWidth_changed)
        if isinstance(self._spacing, State):
            self._spacing.valueChanged.disconnect(self._on_spacing_changed)
        if isinstance(self._onClick, State):
            self._onClick.valueChanged.disconnect(self._on_onClick_changed)
        if isinstance(self._width, State):
            self._width.valueChanged.disconnect(self._on_width_changed)
        if isinstance(self._height, State):
            self._height.valueChanged.disconnect(self._on_height_changed)
        if isinstance(self._borderRadius, State):
            self._borderRadius.valueChanged.disconnect(self._on_borderRadius_changed)


from PySide6.QtWidgets import QHBoxLayout, QFrame, QSpacerItem, QSizePolicy, QPushButton
from PySide6.QtCore import QCoreApplication

class DialogFooter(QFrame):
    """
    DialogFooter
    DialogFooter for dialog

    Args:
        align: "left" | "right"

    Returns:
        new instance of PySyde6.QtWidgets.QFrame
    """
    def __init__(
            self, 
            parent=None,
            align: str = "right"
        ):
        super().__init__(parent)
        self.setLayout(QHBoxLayout())
        self.btn_cancel = QPushButton(QCoreApplication.translate("MainWindow", u"Cancel", None))
        self.btn_cancel.clicked.connect(parent.hide_dialog)
        self.btn_ok = QPushButton(QCoreApplication.translate("MainWindow", u"Ok", None))
        self.btn_ok.clicked.connect(parent.accept)

        if align == "left":
            self.layout().addWidget(self.btn_cancel)
            self.layout().addWidget(self.btn_ok)
            self.layout().addItem(QSpacerItem(448, 20, QSizePolicy.Expanding, QSizePolicy.Minimum))
        else:
            self.layout().addItem(QSpacerItem(448, 20, QSizePolicy.Expanding, QSizePolicy.Minimum))
            self.layout().addWidget(self.btn_cancel)
            self.layout().addWidget(self.btn_ok)


import sys
import uuid
from PySide6.QtWidgets import QFrame, QVBoxLayout, QWidget
from PySide6.QtCore import Qt, QSize
from PySide6.QtGui import QColor

class Item(QWidget):
    def __init__(self, content:QWidget):
        super().__init__()
        # self.setFixedSize(QSize(100, 20))
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        if isinstance(content, QWidget):
            raise TypeError(f"Argument 'content' has incorrect type (expected QWidget, got {type(content)})")
        self.layout().addWidget(content)
        


import sys
from PySide6.QtWidgets import QApplication, QPushButton, QVBoxLayout, QMainWindow, QWidget
from PySide6.QtCore import QObject, Property, Signal, Slot


from src.i18n.use_translation import changeLanguage
from src.i18n.use_translation import changeLanguage as changeLanguageDevMode

from .config_lang import defaultLang, allLangs

changeLanguage(defaultLang)

def setCurrentLanguage(key: str):
    currentLang = allLangs.get(key)
    # print('currentLang_____________________________', currentLang)
    changeLanguage(currentLang)
    changeLanguageDevMode(currentLang)

# class useLocales(QObject):
#     langChanged = Signal()

#     def __init__(self, lang=None):
#         super().__init__()
#         if not lang:
#             self._lang = defaultLang
#         else:
#             self._lang = lang


#     def get_lang(self):
#         return self._lang

#     def set_lang(self, lang):
#         if self._lang != lang:
#             self._lang = lang
#             self.langChanged.emit()

#     def onChangeLang(self, lang):
#         if self._lang != lang:
#             self._lang = lang
#             self.langChanged.emit()

#     lang = Property(str, get_lang, onChangeLang, notify=langChanged)


from PySide6.QtWidgets import QWidget, QVBoxLayout, QLabel
from PySide6.QtCore import Property, Slot

from ...material._common.widget_model import WidgetModel
from qtmui.material.styles import useTheme

from ..qss_name import *

class RHFHidden(QWidget):
    def __init__(
            self,
            name: str,
            id: str = None,
            model: WidgetModel = None,
            helperText: str = "",
            ):
        super().__init__()
        if id is not None:
            self.setObjectName(id)

        self._name = name
        self._model = model
        self._value = None

        if self._model is not None:
            self._value = model.data
            self._model.dataChanged.connect(self.on_model_data_changed)

        self.lbl_helper_text = QLabel(self)
        self.lbl_helper_text.setText(helperText)
        self.lbl_helper_text.setStyleSheet(f"padding-left: 8px;color: {useTheme().palette.text.primary}")
        self.lbl_helper_text.setText("")
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        self.layout().addWidget(self.lbl_helper_text)

    def on_model_data_changed(self):
        self._value = self._model.data
        print(f"Field width name {self._name} has value {self._value}")

    def set_value(self, value=None):
        self._value = value
        print(f"Field width name {self._name} has value {self._value}")

    def set_helper_text(self, text):
        self.lbl_helper_text.setText(text)
        self.lbl_helper_text.setStyleSheet(f"padding-left: 8px;color: {useTheme().palette.text.primary}")

    def set_error_text(self, text):
        self.lbl_helper_text.setText(text)
        self.lbl_helper_text.setStyleSheet(f"padding-left: 8px;color: {useTheme().palette.text.error}")

    @Property(bool)
    def stateSignal(self):
        return self._stateSignal

    @stateSignal.setter
    def stateSignal(self, value):
        self._stateSignal = value
        self._stateSignal.connect(self.state)


    @Slot(object)
    def state(self, state):
        if self._name == state.get("field"):
            if state.get("error"):
                self.set_error_text(str(state.get("message")))


import uuid
from PySide6.QtWidgets import QHBoxLayout, QFrame

styles = """
        #{} {{
            border: {};
            margin: {};
            font-weight: {};
            line-height: {};
            font-size: {};
            font-family: {};
            padding: {};
            border-radius: {};
            color:  {};
            background-color: {};
        }}
        #{}::hover {{
            background-color: {};
        }}

"""


class ToolBar(QFrame):
# class Chip(QWidget):
    """
    props: AdditionalProps & {
        /**
        * The Toolbar children, usually a mixture of `IconButton`, `Button` and `Typography`.
        * The Toolbar is a flex container, allowing flex item properties to be used to lay out the children.
        */
        children?: React.ReactNode;
        /**
        * Override or extend the styles applied to the component.
        */
        classes?: Partial<ToolbarClasses>;
        /**
        * If `true`, disables gutter padding.
        * @default false
        */
        disableGutters?: boolean;
        /**
        * The variant to use.
        * @default 'regular'
        */
        variant?: OverridableStringUnion<'regular' | 'dense', ToolbarPropsVariantOverrides>;
        /**
        * The system prop that allows defining system overrides as well as additional CSS styles.
        */
        sx?: SxProps<Theme>;
    };
    defaultComponent: DefaultComponent;
    """
    def __init__(self,
                variant: str = None,
                disableGutters: bool = None,
                sx: dict = None,
                children: list = None,
                height: int = 50,
                 *args, 
                 **kwargs
                 ):
        super(ToolBar, self).__init__()
        self.setObjectName(str(uuid.uuid4()))
        self.setLayout(QHBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        self.setFixedHeight(height)
        # self.setStyleSheet('''#{}  {{ {} }}'''.format(self.objectName(), "background-color: pink;"))

        self._variant = variant
        self._disableGutters = disableGutters
        self._sx = sx
        self._children = children

         # Thêm các children (nếu có)
        if self._children:
            if not isinstance(self._children, list):
                raise TypeError("children must be type (list)")
            for child in self._children:
                self.layout().addWidget(child)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Animate Box demo
- Box là QFrame có thể chứa children (Box hoặc widget)
- Mỗi Box có AnimQObject để apply initial và play animate
- Container Box (Box with children) sẽ đọc transition.delayChildren và staggerChildren
  và điều phối play() cho các child theo thứ tự (đặt trong showEvent)
"""

import sys
from typing import Optional, Dict, List, Callable, Union, Any
from math import cos, pi

from PySide6.QtCore import *
from PySide6.QtWidgets import *
from PySide6.QtGui import *

from src.components.animate.variants import (
    varFade,
    varZoom,
    varFlip,
    varSlide,
    varScale,
    varBounce,
    varRotate,
    varBgPan,
    varBgColor,
    varBgKenburns,
)

# ----------------------------------------------------------------------
# Shortcut dictionary – giống hệt file getVariant() trong MUI
# ----------------------------------------------------------------------
VARIANT_MAP: Dict[str, Dict[str, Any]] = {
    # Slide
    "slideInUp": varSlide()["inUp"],
    "slideInDown": varSlide()["inDown"],
    "slideInLeft": varSlide()["inLeft"],
    "slideInRight": varSlide()["inRight"],
    "slideOutUp": varSlide()["outUp"],
    "slideOutDown": varSlide()["outDown"],
    "slideOutLeft": varSlide()["outLeft"],
    "slideOutRight": varSlide()["outRight"],

    # Fade
    "fadeIn": varFade()["in"],
    "fadeInUp": varFade()["inUp"],
    "fadeInDown": varFade()["inDown"],
    "fadeInLeft": varFade()["inLeft"],
    "fadeInRight": varFade()["inRight"],
    "fadeOut": varFade()["out"],
    "fadeOutUp": varFade()["outUp"],
    "fadeOutDown": varFade()["outDown"],
    "fadeOutLeft": varFade()["outLeft"],
    "fadeOutRight": varFade()["outRight"],

    # Zoom
    "zoomIn": varZoom({"distance": 0})["in"],
    "zoomInUp": varZoom({"distance": 80})["inUp"],
    "zoomInDown": varZoom({"distance": 80})["inDown"],
    "zoomInLeft": varZoom({"distance": 240})["inLeft"],
    "zoomInRight": varZoom({"distance": 240})["inRight"],
    "zoomOut": varZoom()["out"],
    "zoomOutLeft": varZoom()["outLeft"],
    "zoomOutRight": varZoom()["outRight"],
    "zoomOutUp": varZoom()["outUp"],
    "zoomOutDown": varZoom()["outDown"],

    # Bounce
    "bounceIn": varBounce()["in"],
    "bounceInUp": varBounce()["inUp"],
    "bounceInDown": varBounce()["inDown"],
    "bounceInLeft": varBounce()["inLeft"],
    "bounceInRight": varBounce()["inRight"],
    "bounceOut": varBounce()["out"],
    "bounceOutUp": varBounce()["outUp"],
    "bounceOutDown": varBounce()["outDown"],
    "bounceOutLeft": varBounce()["outLeft"],
    "bounceOutRight": varBounce()["outRight"],

    # Flip
    "flipInX": varFlip()["inX"],
    "flipInY": varFlip()["inY"],
    "flipOutX": varFlip()["outX"],
    "flipOutY": varFlip()["outY"],

    # Scale
    "scaleInX": varScale()["inX"],
    "scaleInY": varScale()["inY"],
    "scaleOutX": varScale()["outX"],
    "scaleOutY": varScale()["outY"],

    # Rotate
    "rotateIn": varRotate()["in"],
    "rotateOut": varRotate()["out"],

    # Background
    "kenburnsTop": varBgKenburns()["top"],
    "kenburnsBottom": varBgKenburns()["bottom"],
    "kenburnsLeft": varBgKenburns()["left"],
    "kenburnsRight": varBgKenburns()["right"],

    "panTop": varBgPan()["top"],
    "panBottom": varBgPan()["bottom"],
    "panLeft": varBgPan()["left"],
    "panRight": varBgPan()["right"],

    "color2x": varBgColor(),
    "color3x": varBgColor({"colors": ['#19dcea', '#b22cff', '#ea2222']}),
    "color4x": varBgColor({"colors": ['#19dcea', '#b22cff', '#ea2222', '#f5be10']}),
    "color5x": varBgColor({"colors": ['#19dcea', '#b22cff', '#ea2222', '#f5be10', '#3bd80d']}),
}


# ----------------------------------------------------------------------
# Hàm tiện ích – giống hệt getVariant(variant = 'slideInUp')
# ----------------------------------------------------------------------
def getVariant(variant: str = "slideInUp") -> Dict[str, Any]:
    """
    Dùng chỉ bằng tên chuỗi – cực tiện!

    Ví dụ:
        getVariant("zoomInUp")
        getVariant("bounceIn")
        getVariant("kenburnsTop")
    """
    return VARIANT_MAP.get(variant, varSlide()["inUp"])  # fallback về slideInUp nếu không tìm thấy




from typing import Any, Generic, TypeVar, Optional

T = TypeVar('T')

class RefObject(Generic[T]):
    """
    A generic class that simulates React's useRef.
    It holds a mutable value that can be accessed and modified.
    """
    def __init__(self, current: Optional[T] = None):
        self.current: Optional[T] = current

    def __repr__(self):
        return f"RefObject(current={self.current})"

def useRef(initial_value: Optional[T] = None) -> RefObject[T]:
    """
    A function that simulates React's useRef hook.
    It returns a RefObject that holds a mutable value.
    """
    return RefObject(initial_value)


import uuid
from typing import Optional, Union, Dict, List, Callable
from PySide6.QtWidgets import QWidget, QHBoxLayout, QSizePolicy, QFrame
from PySide6.QtCore import Qt, Signal
from PySide6.QtGui import QFocusEvent, QKeyEvent, QMouseEvent
from qtmui.hooks import State, useEffect
from qtmui.material.styles import useTheme
from ..utils.validate_params import _validate_param
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from ..typography import Typography
from .menu_item_base import MenuItemBase
from qtmui.i18n.use_translation import translate, i18n

class MenuItem(MenuItemBase):
    """
    A component that renders a menu item, styled like Material-UI MenuItem.

    The `MenuItem` component is used within a `Menu` to represent an actionable item, with support for
    selection, dense layout, dividers, and focus management. It integrates with `Menu`, `ListSubheader`,
    and `Masonry` components in the `qtmui` framework, retaining all existing parameters and adding
    support for additional MUI props.

    Parameters
    ----------
    alignItems : State or str, optional
        Alignment of items in the layout ("flex-start", "center", "flex-end"). Default is "center".
        Can be a `State` object for dynamic updates.
    autoFocus : State or bool, optional
        If True, the item is focused during the first mount or when changed to True. Default is False.
        Can be a `State` object for dynamic updates.
    children : State, str, QWidget, or List[QWidget], optional
        The content of the component. Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    component : State, type, str, or None, optional
        The component used for the root node (e.g., QFrame). Default is None (uses QFrame).
        Can be a `State` object for dynamic updates.
    dense : State or bool, optional
        If True, uses compact padding. Default is False, inherited from parent Menu.
        Can be a `State` object for dynamic updates.
    disabled : State or bool, optional
        If True, the item is disabled. Default is False.
        Can be a `State` object for dynamic updates.
    disableGutters : State or bool, optional
        If True, removes left and right padding. Default is False.
        Can be a `State` object for dynamic updates.
    divider : State or bool, optional
        If True, adds a 1px border at the bottom. Default is False.
        Can be a `State` object for dynamic updates.
    focusVisibleClassName : State or str, optional
        Class name applied when the item gains focus via keyboard. Default is None.
        Can be a `State` object for dynamic updates.
    key : State, str, int, or None, optional
        Identifier for the item. Default is None.
        Can be a `State` object for dynamic updates.
    minHeight : State or int, optional
        Minimum height of the item in pixels. Default is None.
        Can be a `State` object for dynamic updates.
    onChange : State or Callable, optional
        Callback fired when the selection state changes. Default is None.
        Can be a `State` object for dynamic updates.
    onClick : State or Callable, optional
        Callback fired when the item is clicked. Default is None.
        Can be a `State` object for dynamic updates.
    onFocus : State or Callable, optional
        Callback fired when the item gains focus. Default is None.
        Can be a `State` object for dynamic updates.
    onKeyDown : State or Callable, optional
        Callback fired when a key is pressed. Default is None.
        Can be a `State` object for dynamic updates.
    selected : State or bool, optional
        If True, the item is selected. Default is False.
        Can be a `State` object for dynamic updates.
    sx : State, list, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    size : State or str, optional
        Size of the item ("small", "medium", "large"). Default is "medium".
        Can be a `State` object for dynamic updates.
    parent : QWidget or None, optional
        Parent widget. Default is None.
    selectedKey : State or None, optional
        State containing the selected key(s). Default is None.
        Can be a `State` object for dynamic updates.
    text : State or str, optional
        Text content of the item, used if children is None. Default is ''.
        Can be a `State` object for dynamic updates.
    textAlign : State or str, optional
        Text alignment ("left", "center", "right"). Default is "left".
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `MenuItemBase` class,
        supporting props of the ButtonBase component (e.g., style, className).

    Signals
    -------
    selectedChange : Signal
        Emitted when the selection state changes.
    clicked : Signal
        Emitted when the item is clicked.
    focusReceived : Signal
        Emitted when the item gains focus.
    keyPressed : Signal
        Emitted when a key is pressed.

    Notes
    -----
    - All existing parameters from the previous implementation are retained.
    - Props of the ButtonBase component are supported via `**kwargs` and explicit parameters (`disabled`, `onClick`, `onFocus`, `onKeyDown`).
    - The `dense` prop is inherited from the parent `Menu` if not explicitly set.
    - The `focusVisibleClassName` prop applies a class when focused via keyboard, emulating CSS `:focus-visible`.
    - MUI classes applied: `MuiMenuItem-root`, `MuiMenuItem-selected`, `MuiMenuItem-dense`, 
      `MuiMenuItem-divider`, `MuiMenuItem-guttersDisabled`.
    - Integrates with `Menu`, `ListSubheader`, and `Masonry` components for consistent styling and layout.

    Demos:
    - MenuItem: https://qtmui.com/material-ui/qtmui-menuitem/

    API Reference:
    - MenuItem API: https://qtmui.com/material-ui/api/menu-item/
    """

    selectedChange = Signal(object)
    clicked = Signal()
    focusReceived = Signal()
    keyPressed = Signal()

    VALID_ALIGN_ITEMS = ["flex-start", "center", "flex-end"]
    VALID_SIZES = ["small", "medium", "large"]
    VALID_TEXT_ALIGNS = ["left", "center", "right"]

    def __init__(
        self,
        alignItems: Union[State, str] = "center",
        autoFocus: Union[State, bool] = False,
        children: Optional[Union[State, str, QWidget, List[QWidget]]] = None,
        classes: Optional[Union[State, Dict]] = None,
        component: Optional[Union[State, type, str]] = None,
        dense: Union[State, bool] = False,
        disabled: Union[State, bool] = False,
        disableGutters: Union[State, bool] = False,
        divider: Union[State, bool] = False,
        focusVisibleClassName: Optional[Union[State, str]] = None,
        key: Optional[Union[State, str, int]] = None,
        minHeight: Optional[Union[State, int]] = None,
        onChange: Optional[Union[State, Callable]] = None,
        onClick: Optional[Union[State, Callable]] = None,
        onFocus: Optional[Union[State, Callable]] = None,
        onKeyDown: Optional[Union[State, Callable]] = None,
        selected: Union[State, bool] = False,
        sx: Optional[Union[State, List, Dict, Callable, str]] = None,
        size: Union[State, str] = "medium",
        parent: Optional[QWidget] = None,
        selectedKeys: Optional[State] = None,
        text: Union[State, str] = "",
        textAlign: Union[State, str] = "left",
        **kwargs
    ):
        super().__init__(parent=parent, **kwargs)
        self.setObjectName(f"MenuItem-{str(uuid.uuid4())}")

        self.theme = useTheme()
        self._widget_references = []
        self._keyboard_focus = False

        # Set properties with validation
        self._set_alignItems(alignItems)
        self._set_autoFocus(autoFocus)
        self._set_children(children)
        self._set_classes(classes)
        self._set_component(component)
        self._set_dense(dense)
        self._set_disabled(disabled)
        self._set_disableGutters(disableGutters)
        self._set_divider(divider)
        self._set_focusVisibleClassName(focusVisibleClassName)
        self._set_key(key)
        self._set_minHeight(minHeight)
        self._set_onChange(onChange)
        self._set_onClick(onClick)
        self._set_onFocus(onFocus)
        self._set_onKeyDown(onKeyDown)
        self._set_selected(selected)
        self._set_sx(sx)
        self._set_size(size)
        self._set_selectedKey(selectedKeys)
        self._set_text(text)
        self._set_textAlign(textAlign)

        self.__init_ui()

    # Setter and Getter methods
    @_validate_param(
        file_path="qtmui.material.menuitem",
        param_name="alignItems",
        supported_signatures=Union[State, str],
        valid_values=VALID_ALIGN_ITEMS
    )
    def _set_alignItems(self, value):
        """Assign value to alignItems."""
        self._alignItems = value

    def _get_alignItems(self):
        """Get the alignItems value."""
        return self._alignItems.value if isinstance(self._alignItems, State) else self._alignItems

    @_validate_param(file_path="qtmui.material.menuitem", param_name="autoFocus", supported_signatures=Union[State, bool])
    def _set_autoFocus(self, value):
        """Assign value to autoFocus."""
        self._autoFocus = value
        if self._get_autoFocus():
            self.setFocus()

    def _get_autoFocus(self):
        """Get the autoFocus value."""
        return self._autoFocus.value if isinstance(self._autoFocus, State) else self._autoFocus

    # @_validate_param(
    #     file_path="qtmui.material.menuitem",
    #     param_name="children",
    #     supported_signatures=Union[State, str, QWidget, List, type(None)]
    # )
    def _set_children(self, value):
        """Assign value to children and store references."""
        self._children = value

    def _get_children(self):
        """Get the children value."""
        children = self._children.value if isinstance(self._children, State) else self._children
        if isinstance(children, str):
            return [Typography(text=children, variant="body1")]
        return children if isinstance(children, list) else [children] if isinstance(children, QWidget) else []

    @_validate_param(file_path="qtmui.material.menuitem", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        """Assign value to classes."""
        self._classes = value

    def _get_classes(self):
        """Get the classes value."""
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.menuitem", param_name="component", supported_signatures=Union[State, type, str, type(None)])
    def _set_component(self, value):
        """Assign value to component."""
        self._component = value

    def _get_component(self):
        """Get the component value."""
        return self._component.value if isinstance(self._component, State) else self._component or QFrame

    @_validate_param(file_path="qtmui.material.menuitem", param_name="dense", supported_signatures=Union[State, bool])
    def _set_dense(self, value):
        """Assign value to dense."""
        self._dense = value

    def _get_dense(self):
        """Get the dense value."""
        dense = self._dense.value if isinstance(self._dense, State) else self._dense
        # Inherit from parent Menu if available
        parent = self.parent()
        if hasattr(parent, "_get_dense") and not dense:
            return parent._get_dense()
        return dense

    @_validate_param(file_path="qtmui.material.menuitem", param_name="disabled", supported_signatures=Union[State, bool])
    def _set_disabled(self, value):
        """Assign value to disabled."""
        self._disabled = value
        self.setDisabled(self._get_disabled())

    def _get_disabled(self):
        """Get the disabled value."""
        return self._disabled.value if isinstance(self._disabled, State) else self._disabled

    @_validate_param(file_path="qtmui.material.menuitem", param_name="disableGutters", supported_signatures=Union[State, bool])
    def _set_disableGutters(self, value):
        """Assign value to disableGutters."""
        self._disableGutters = value

    def _get_disableGutters(self):
        """Get the disableGutters value."""
        return self._disableGutters.value if isinstance(self._disableGutters, State) else self._disableGutters

    @_validate_param(file_path="qtmui.material.menuitem", param_name="divider", supported_signatures=Union[State, bool])
    def _set_divider(self, value):
        """Assign value to divider."""
        self._divider = value

    def _get_divider(self):
        """Get the divider value."""
        return self._divider.value if isinstance(self._divider, State) else self._divider

    @_validate_param(
        file_path="qtmui.material.menuitem",
        param_name="focusVisibleClassName",
        supported_signatures=Union[State, str, type(None)],
        validator=lambda x: isinstance(x, str) and x.strip() != "" if x else True
    )
    def _set_focusVisibleClassName(self, value):
        """Assign value to focusVisibleClassName."""
        self._focusVisibleClassName = value

    def _get_focusVisibleClassName(self):
        """Get the focusVisibleClassName value."""
        return self._focusVisibleClassName.value if isinstance(self._focusVisibleClassName, State) else self._focusVisibleClassName

    @_validate_param(file_path="qtmui.material.menuitem", param_name="key", supported_signatures=Union[State, str, int, type(None)])
    def _set_key(self, value):
        """Assign value to key."""
        self._key = value

    def _get_key(self):
        """Get the key value."""
        return self._key.value if isinstance(self._key, State) else self._key

    @_validate_param(
        file_path="qtmui.material.menuitem",
        param_name="minHeight",
        supported_signatures=Union[State, int, type(None)],
        validator=lambda x: x > 0 if isinstance(x, int) else True
    )
    def _set_minHeight(self, value):
        """Assign value to minHeight."""
        self._minHeight = value

    def _get_minHeight(self):
        """Get the minHeight value."""
        return self._minHeight.value if isinstance(self._minHeight, State) else self._minHeight

    @_validate_param(file_path="qtmui.material.menuitem", param_name="onChange", supported_signatures=Union[State, Callable, type(None)])
    def _set_onChange(self, value):
        """Assign value to onChange."""
        self._onChange = value
        if value:
            self.selectedChange.connect(self._get_onChange)

    def _get_onChange(self):
        """Get the onChange value."""
        return self._onChange.value if isinstance(self._onChange, State) else self._onChange

    @_validate_param(file_path="qtmui.material.menuitem", param_name="onClick", supported_signatures=Union[State, Callable, type(None)])
    def _set_onClick(self, value):
        """Assign value to onClick."""
        self._onClick = value

    def _get_onClick(self):
        """Get the onClick value."""
        return self._onClick.value if isinstance(self._onClick, State) else self._onClick

    @_validate_param(file_path="qtmui.material.menuitem", param_name="onFocus", supported_signatures=Union[State, Callable, type(None)])
    def _set_onFocus(self, value):
        """Assign value to onFocus."""
        self._onFocus = value

    def _get_onFocus(self):
        """Get the onFocus value."""
        return self._onFocus.value if isinstance(self._onFocus, State) else self._onFocus

    @_validate_param(file_path="qtmui.material.menuitem", param_name="onKeyDown", supported_signatures=Union[State, Callable, type(None)])
    def _set_onKeyDown(self, value):
        """Assign value to onKeyDown."""
        self._onKeyDown = value

    def _get_onKeyDown(self):
        """Get the onKeyDown value."""
        return self._onKeyDown.value if isinstance(self._onKeyDown, State) else self._onKeyDown

    @_validate_param(file_path="qtmui.material.menuitem", param_name="selected", supported_signatures=Union[State, bool])
    def _set_selected(self, value):
        """Assign value to selected."""
        self._selected = value
        # self.setProperty("selected", str(self._get_selected()).lower())
        # self._set_stylesheet()

    def _get_selected(self):
        """Get the selected value."""
        selected = self._selected.value if isinstance(self._selected, State) else self._selected
        if self._get_selectedKey() is not None:
            keys = self._get_selectedKey()
            key = self._get_key()
            if key is not None:
                return key in (keys if isinstance(keys, list) else [keys])
        return selected

    @_validate_param(file_path="qtmui.material.menuitem", param_name="sx", supported_signatures=Union[State, List, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        """Assign value to sx."""
        self._sx = value

    def _get_sx(self):
        """Get the sx value."""
        return self._sx.value if isinstance(self._sx, State) else self._sx

    @_validate_param(
        file_path="qtmui.material.menuitem",
        param_name="size",
        supported_signatures=Union[State, str],
        valid_values=VALID_SIZES
    )
    def _set_size(self, value):
        """Assign value to size."""
        self._size = value

    def _get_size(self):
        """Get the size value."""
        return self._size.value if isinstance(self._size, State) else self._size

    @_validate_param(file_path="qtmui.material.menuitem", param_name="selectedKey", supported_signatures=Union[State, type(None)])
    def _set_selectedKey(self, value):
        """Assign value to selectedKey."""
        self._selectedKeys = value

    def _get_selectedKey(self):
        """Get the selectedKey value."""
        return self._selectedKeys.value if isinstance(self._selectedKeys, State) else self._selectedKeys

    @_validate_param(file_path="qtmui.material.menuitem", param_name="text", supported_signatures=Union[State, str])
    def _set_text(self, value):
        """Assign value to text."""
        self._text = value

    def _get_text(self):
        """Get the text value."""
        return self._text.value if isinstance(self._text, State) else self._text

    @_validate_param(
        file_path="qtmui.material.menuitem",
        param_name="textAlign",
        supported_signatures=Union[State, str],
        valid_values=VALID_TEXT_ALIGNS
    )
    def _set_textAlign(self, value):
        """Assign value to textAlign."""
        self._textAlign = value

    def _get_textAlign(self):
        """Get the textAlign value."""
        return self._textAlign.value if isinstance(self._textAlign, State) else self._textAlign


    def __init_ui(self):
        # self.setObjectName("PyMenuItem")
        self.setObjectName(str(uuid.uuid4()))


        self.setDisabled(self._disabled)

        self.setCursor(Qt.CursorShape.PointingHandCursor)

        self.setText(self._text)

        if self._textAlign in ["left", "center", "right"]:
            self._textAlign = f"text-align: {self._textAlign};"

        if self._minHeight:
            self.setMinimumHeight(self._minHeight)

        if self._selectedKeys:
            if isinstance(self._selectedKeys, State):
                self._selectedKeys.valueChanged.connect(self._set_selected)

        if self._children:
            self.setLayout(QHBoxLayout())
            self.layout().setContentsMargins(6,6,6,6)
            self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Maximum)

            if isinstance(self._children, QWidget):
                self.layout().addWidget(self._children)
            elif isinstance(self._children, str):
                self.layout().addWidget(Typography(text=self._children))
            elif isinstance(self._children, list):
                for child in self._children:
                    if isinstance(child, str):
                        self.layout().addWidget(Typography(text=child))
                    elif isinstance(child, QWidget):
                        self.layout().addWidget(child)

        if self._onChange:
            self.selectedChange.connect(self._onChange)

        theme = useTheme()
        useEffect(
            self._set_stylesheet,
            [theme.state]
        )
        self._set_stylesheet()

        i18n.langChanged.connect(self.retranslateUi)
        self.retranslateUi()

    def retranslateUi(self):
        pass

    def _set_stylesheet(self):
        theme = useTheme()
        component_styles = theme.components

        PyMenuItem_root = component_styles[f"PyMenuItem"].get("styles")["root"]
        PyMenuItem_root_qss = get_qss_style(PyMenuItem_root)
        PyMenuItem_root_slot_hover_qss = get_qss_style(PyMenuItem_root["slots"]["hover"])
        PyMenuItem_root_slot_selected_qss = get_qss_style(PyMenuItem_root["slots"]["selected"])
        PyMenuItem_root_slot_selected_hover_qss = get_qss_style(PyMenuItem_root["slots"]["selected"]["hover"])
        PyMenuItem_root_prop_notLastOfType_qss = get_qss_style(PyMenuItem_root["props"]["notLastOfType"])

        PyMenuItem_checkbox_qss = get_qss_style(component_styles[f"PyMenuItem"].get("styles")["checkbox"])
        PyMenuItem_autocomplete_qss = get_qss_style(component_styles[f"PyMenuItem"].get("styles")["autocomplete"])
        PyMenuItem_divider_qss = get_qss_style(component_styles[f"PyMenuItem"].get("styles")["divider"])


        self.setStyleSheet(
            f"""
                #{self.objectName()} {{
                    {PyMenuItem_root_qss}
                }}
                #{self.objectName()}::hover {{
                    {PyMenuItem_root_slot_hover_qss}
                }}
                #{self.objectName()}[selected=true] {{
                    {PyMenuItem_root_slot_selected_qss}
                }}
                #{self.objectName()}[selected=true]::hover {{
                    {PyMenuItem_root_slot_selected_hover_qss}
                }}
                #{self.objectName()}[notLastOfType=true]::hover {{
                    {PyMenuItem_root_prop_notLastOfType_qss}
                }}
                #{self.objectName()} #MuiCheckbox {{
                    {PyMenuItem_checkbox_qss}
                }}
                #{self.objectName()} #MuiAutocomplete {{
                    {PyMenuItem_autocomplete_qss}
                }}
                #{self.objectName()} #PyDivider {{
                    {PyMenuItem_divider_qss}
                }}
            """
        )


    def _set_on_changed(self, onChange: Callable):
        self._onChange = onChange
        self.selectedChange.connect(self._onChange)

    def _set_selected_keys(self, selectedKeys: State):
        self._selectedKeys = selectedKeys
        self._selectedKeys.valueChanged.connect(self._set_selected)
        self._set_selected(selectedKeys.value)

    def _set_selected(self, keys):
        if isinstance(keys, list):
            if self._key in keys:
                self.setProperty("selected", "true")
            else:
                self.setProperty("selected", "false")
        else:
            if self._key == keys:
                self.setProperty("selected", "true")
            else:
                self.setProperty("selected", "false")
        self._set_stylesheet()


    def mouseReleaseEvent(self, event):
        if self._onClick:
            if self._key is not None:
                self._onClick(self._key)
            else:
                self._onClick()
        self.selectedChange.emit(self._key)
        # self.setProperty("selected", "true")
        # self._set_stylesheet()
        return super().mouseReleaseEvent(event)



    


import asyncio
from typing import Optional, Union, Callable, Any, Dict, List
import uuid

from qtmui.hooks import State, useEffect
from PySide6.QtWidgets import QWidget, QHBoxLayout, QSizePolicy, QPushButton
from PySide6.QtCore import Qt, QSize, QTimer

from qtmui.material.spacer import HSpacer
from qtmui.material.py_svg_widget import PySvgWidget
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style

from qtmui.material.styles import useTheme
from qtmui.i18n.use_translation import translate, i18n
from ..widget_base import PyWidgetBase
from ..utils.validate_params import _validate_param

class Breadcrumbs(QWidget, PyWidgetBase):
    """
    A component that displays a list of breadcrumbs with customizable separators and collapse behavior.

    The `Breadcrumbs` component is used to show navigation paths, with support for collapsing
    items when the number of breadcrumbs exceeds a specified maximum. It supports all props
    of the Material-UI `Breadcrumbs` component, as well as props of the `Typography` component
    via `**kwargs`.

    Parameters
    ----------
    children : State or List[Any], optional
        The content of the Breadcrumbs, typically a list of `Typography` or other components.
        Default is None. Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    component : State or Any, optional
        Component used for the root node (e.g., HTML element or custom component).
        Default is None. Can be a `State` object for dynamic updates.
    expandText : State or str, optional
        Label for the expand button, used when breadcrumbs are collapsed.
        Default is "Show path". Can be a `State` object for dynamic updates.
    itemsAfterCollapse : State or int, optional
        Number of items to show after the ellipsis when collapsed. Default is 1.
        Can be a `State` object for dynamic updates.
    itemsBeforeCollapse : State or int, optional
        Number of items to show before the ellipsis when collapsed. Default is 1.
        Can be a `State` object for dynamic updates.
    maxItems : State or int, optional
        Maximum number of breadcrumbs to display before collapsing. Default is 8.
        Can be a `State` object for dynamic updates.
    separator : State, str, Any, or None, optional
        Custom separator between breadcrumbs. Can be a string, component, or other node.
        Default is "/". Can be a `State` object for dynamic updates.
    slotProps : State or dict, optional
        Props for each slot (e.g., collapsedIcon). Default is None.
        Can be a `State` object for dynamic updates.
    slots : State or dict, optional
        Components for each slot (e.g., CollapsedIcon). Default is None.
        Can be a `State` object for dynamic updates.
    sx : State, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QWidget` class, supporting
        props of the `Typography` component (e.g., variant, color).

    Attributes
    ----------
    VALID_SEPARATOR_TYPES : list[str]
        Valid types for the `separator` parameter: ["str", "Typography", "PyToolButton", "custom"].

    Notes
    -----
    - Props of the `Typography` component are supported via `**kwargs` (e.g., `variant`, `color`).
    - When the number of children exceeds `maxItems`, the component collapses to show
      `itemsBeforeCollapse` items, an ellipsis, and `itemsAfterCollapse` items.
    - The `separator` can be a string, `Typography`, `PyToolButton`, or custom component.

    Demos:
    - Breadcrumbs: https://qtmui.com/material-ui/qtmui-breadcrumbs/

    API Reference:
    - Breadcrumbs API: https://qtmui.com/material-ui/api/breadcrumbs/
    """

    VALID_SEPARATOR_TYPES = ["str", "Typography", "PyToolButton", "custom"]

    def __init__(
        self,
        children: Optional[Union[State, List[Any]]] = None,
        classes: Optional[Union[State, Dict]] = None,
        component: Optional[Union[State, Any]] = None,
        expandText: Union[State, str] = "Show path",
        itemsAfterCollapse: Union[State, int] = 1,
        itemsBeforeCollapse: Union[State, int] = 1,
        maxItems: Union[State, int] = 8,
        separator: QWidget = None,
        slotProps: Optional[Union[State, Dict]] = None,
        slots: Optional[Union[State, Dict]] = None,
        sx: Optional[Union[State, Dict, Callable, str]] = None,
        **kwargs
    ):
        super().__init__(**kwargs)

        # Set properties with validation
        self._set_children(children)
        self._set_classes(classes)
        self._set_component(component)
        self._set_expandText(expandText)
        self._set_itemsAfterCollapse(itemsAfterCollapse)
        self._set_itemsBeforeCollapse(itemsBeforeCollapse)
        self._set_maxItems(maxItems)
        self._set_separator(separator)
        self._set_slotProps(slotProps)
        self._set_slots(slots)
        self._set_sx(sx)

        self._is_collapsed = False  # Track collapse state

        self._init_ui()
        self._set_stylesheet()

        # i18n.langChanged.connect(self.retranslateUi)
        self.theme = useTheme()
        useEffect(
            self._set_stylesheet,
            [self.theme.state]
        )
        self.destroyed.connect(self._on_destroyed)

    # Setter and Getter methods for all parameters
    # @_validate_param(file_path="qtmui.material.breadcrumbs", param_name="children", supported_signatures=Union[State, List[Any], type(None)])
    def _set_children(self, value):
        self._children = value

    def _get_children(self):
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.breadcrumbs", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        self._classes = value

    def _get_classes(self):
        return self._classes.value if isinstance(self._classes, State) else self._classes

    # @_validate_param(file_path="qtmui.material.breadcrumbs", param_name="component", supported_signatures=Union[State, Any, type(None)])
    def _set_component(self, value):
        self._component = value

    def _get_component(self):
        return self._component.value if isinstance(self._component, State) else self._component

    @_validate_param(file_path="qtmui.material.breadcrumbs", param_name="expandText", supported_signatures=Union[State, str])
    def _set_expandText(self, value):
        self._expandText = value

    def _get_expandText(self):
        return self._expandText.value if isinstance(self._expandText, State) else self._expandText

    @_validate_param(file_path="qtmui.material.breadcrumbs", param_name="itemsAfterCollapse", supported_signatures=Union[State, int])
    def _set_itemsAfterCollapse(self, value):
        if value < 0:
            raise ValueError("itemsAfterCollapse must be non-negative")
        self._itemsAfterCollapse = value

    def _get_itemsAfterCollapse(self):
        return self._itemsAfterCollapse.value if isinstance(self._itemsAfterCollapse, State) else self._itemsAfterCollapse

    @_validate_param(file_path="qtmui.material.breadcrumbs", param_name="itemsBeforeCollapse", supported_signatures=Union[State, int])
    def _set_itemsBeforeCollapse(self, value):
        if value < 0:
            raise ValueError("itemsBeforeCollapse must be non-negative")
        self._itemsBeforeCollapse = value

    def _get_itemsBeforeCollapse(self):
        return self._itemsBeforeCollapse.value if isinstance(self._itemsBeforeCollapse, State) else self._itemsBeforeCollapse

    @_validate_param(file_path="qtmui.material.breadcrumbs", param_name="maxItems", supported_signatures=Union[State, int])
    def _set_maxItems(self, value):
        if value < (self._get_itemsBeforeCollapse() + self._get_itemsAfterCollapse()):
            raise ValueError("maxItems must be at least itemsBeforeCollapse + itemsAfterCollapse")
        self._maxItems = value

    def _get_maxItems(self):
        return self._maxItems.value if isinstance(self._maxItems, State) else self._maxItems

    # @_validate_param(file_path="qtmui.material.breadcrumbs", param_name="separator", supported_signatures=Union[State, str, Any, type(None)])
    def _set_separator(self, value):
        self._separator = value

    def _get_separator(self):
        return self._separator.value if isinstance(self._separator, State) else self._separator

    @_validate_param(file_path="qtmui.material.breadcrumbs", param_name="slotProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_slotProps(self, value):
        self._slotProps = value or {}

    def _get_slotProps(self):
        return self._slotProps.value if isinstance(self._slotProps, State) else self._slotProps

    @_validate_param(file_path="qtmui.material.breadcrumbs", param_name="slots", supported_signatures=Union[State, Dict, type(None)])
    def _set_slots(self, value):
        self._slots = value or {}

    def _get_slots(self):
        return self._slots.value if isinstance(self._slots, State) else self._slots

    @_validate_param(file_path="qtmui.material.breadcrumbs", param_name="sx", supported_signatures=Union[State, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        self._sx = value

    def _get_sx(self):
        return self._sx.value if isinstance(self._sx, State) else self._sx
    def slot_set_stylesheet(self, value=None):
        self._set_stylesheet()

    def _init_ui(self):
        self.setObjectName(str(uuid.uuid4()))

        self._children = self.insert_between_elements(self._children)
        self.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Maximum)
        self.setLayout(QHBoxLayout())
        self.layout().setAlignment(Qt.AlignmentFlag.AlignVCenter | Qt.AlignmentFlag.AlignLeft)
        self.layout().setContentsMargins(0,0,0,0)
        
        if isinstance(self._children, list):
            for index, widget in enumerate(self._children, 0):
                if index != len(self._children) - 1:
                    widget.setParent(self)
                    widget.setCursor(Qt.PointingHandCursor)
                self.layout().addWidget(widget)
                
        self.layout().addWidget(HSpacer())
        
    def _set_stylesheet(self, component_styled=None):
        self.theme = useTheme()

        if not component_styled:
            component_styled = self.theme.components

        PyBreadcrumbs_root = component_styled[f"PyBreadcrumbs"].get("styles")["root"]
        PyBreadcrumbs_root_qss = get_qss_style(PyBreadcrumbs_root)

        sx_qss = ""
        if self._sx:
            if isinstance(self._sx, dict):
                sx_qss = get_qss_style(self._sx, class_name=f"#{self.objectName()}")
            elif isinstance(self._sx, Callable):
                sx = self._sx()
                if isinstance(sx, dict):
                    sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
                elif isinstance(sx, str):
                    sx_qss = sx
            elif isinstance(self._sx, str) and self._sx != "":
                sx_qss = self._sx

        self.setStyleSheet(
            f"""
                #{self.objectName()} {{
                    {PyBreadcrumbs_root_qss}
                }}

                {sx_qss}

            """
        )

    def insert_between_elements(self, lst):
        result = [elem for i in range(len(lst)) for elem in (lst[i], PySvgWidget(key="iconoir:slash") if self._separator == None else self._separator() if callable(self._separator) else None)][:-1]
        return result


from PySide6.QtWidgets import QApplication, QWidget, QLabel, QVBoxLayout, QPushButton
from PySide6.QtGui import QPixmap, QPainter, QPen, QBrush
from PySide6.QtCore import Qt, QSize

def widget_to_pixmap(widget):
    """Chuyển đổi một QWidget thành QPixmap."""
    # Tạo một đối tượng QPixmap với kích thước của widget
    pixmap = QPixmap(widget.size())
    
    # Bắt đầu vẽ nội dung của widget lên pixmap
    widget.render(pixmap)
    
    return pixmap

class ExampleWidget(QWidget):
    def __init__(self):
        super().__init__()
        
        # Tạo layout và label đơn giản
        layout = QVBoxLayout(self)
        label = QPushButton("Hello, QLabel in QWidget!")
        label.setStyleSheet('background: red;')
        layout.addWidget(label)
        
        self.setLayout(layout)
        self.setFixedSize(QSize(200, 40))

class PixmapWidget(QWidget):
    def __init__(self):
        super().__init__()

        
        self.setFixedSize(QSize(200, 40))
        

    def paintEvent(self, event):
        # Chuyển đổi widget thành QPixmap
        pixmap = widget_to_pixmap(ExampleWidget())
        # Lưu QPixmap ra file ảnh PNG để kiểm tra
        # pixmap.save("widget_capture.png", "PNG")
        scaled_pixmap = pixmap.scaled(self.size(), Qt.AspectRatioMode.KeepAspectRatioByExpanding, Qt.TransformationMode.SmoothTransformation)

        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)  

        brush = QBrush()
        brush.setStyle(Qt.Dense2Pattern)
        brush.setTexture(pixmap)
        painter.setPen(Qt.NoPen)
        painter.setBrush(brush)

        painter.drawRoundedRect(self.rect(), 200, 40)

        super().paintEvent(event)


if __name__ == "__main__":
    app = QApplication([])

    # Tạo một widget ví dụ
    widget = PixmapWidget()
    widget.show()



    app.exec()



from PySide6.QtWidgets import (QMenu, QVBoxLayout)

class CustomMenu(QMenu):
    def __init__(
                self,
                parent = None,
                context = None,
                content: object = None,
                configs: object = None
                ):
        super().__init__(parent)
        self._context = context
        
        self._content = content
        self._configs = configs
        # self.setMinimumSize(400, 500)
        self.initUI()

    def initUI(self):
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)
        if self._content:
            self.layout().addWidget(self._content)
        # else:
        #     for config in self._configs:
        #         self.layout().addWidget(Button(text=config.get("label"), variant="soft", text="inherit", fullWidth=True, onClick=config.get("label")))





from PySide6.QtCore import QEasingCurve

def chooseEasing(ease_data):
    """Choose QEasingCurve from ease_data. If ease_data is a list (bezier),
    fallback to OutCubic for now. If it's a string, attempt to map.
    """
    # if given as list/tuple -> fallback
    if isinstance(ease_data, (list, tuple)):
        return QEasingCurve(QEasingCurve.Type.OutCubic)
    # if string -> try map common ones
    if isinstance(ease_data, str):
        # minimal mapping, extend if needed
        easing_map = {
            'linear': QEasingCurve.Type.Linear,
            'inQuad': QEasingCurve.Type.InQuad,
            'outQuad': QEasingCurve.Type.OutQuad,
            'inOutQuad': QEasingCurve.Type.InOutQuad,
            'outInQuad': QEasingCurve.Type.OutInQuad,
            'inCubic': QEasingCurve.Type.InCubic,
            'outCubic': QEasingCurve.Type.OutCubic,
            'inOutCubic': QEasingCurve.Type.InOutCubic,
            'outInCubic': QEasingCurve.Type.OutInCubic,
            'inQuart': QEasingCurve.Type.InQuart,
            'outQuart': QEasingCurve.Type.OutQuart,
            'inOutQuart': QEasingCurve.Type.InOutQuart,
            'outInQuart': QEasingCurve.Type.OutInQuart,
            'inQuint': QEasingCurve.Type.InQuint,
            'outQuint': QEasingCurve.Type.OutQuint,
            'inOutQuint': QEasingCurve.Type.InOutQuint,
            'outInQuint': QEasingCurve.Type.OutInQuint,
            'inSine': QEasingCurve.Type.InSine,
            'outSine': QEasingCurve.Type.OutSine,
            'inOutSine': QEasingCurve.Type.InOutSine,
            'outInSine': QEasingCurve.Type.OutInSine,
            'inExpo': QEasingCurve.Type.InExpo,
            'outExpo': QEasingCurve.Type.OutExpo,
            'inOutExpo': QEasingCurve.Type.InOutExpo,
            'outInExpo': QEasingCurve.Type.OutInExpo,
            'inCirc': QEasingCurve.Type.InCirc,
            'outCirc': QEasingCurve.Type.OutCirc,
            'inOutCirc': QEasingCurve.Type.InOutCirc,
            'outInCirc': QEasingCurve.Type.OutInCirc,
            'inElastic': QEasingCurve.Type.InElastic,
            'outElastic': QEasingCurve.Type.OutElastic,
            'inOutElastic': QEasingCurve.Type.InOutElastic,
            'outInElastic': QEasingCurve.Type.OutInElastic,
            'inBack': QEasingCurve.Type.InBack,
            'outBack': QEasingCurve.Type.OutBack,
            'inOutBack': QEasingCurve.Type.InOutBack,
            'outInBack': QEasingCurve.Type.OutInBack,
            'inBounce': QEasingCurve.Type.InBounce,
            'outBounce': QEasingCurve.Type.OutBounce,
            'inOutBounce': QEasingCurve.Type.InOutBounce,
            'outInBounce': QEasingCurve.Type.OutInBounce,
            # fallback cho các easing khác
            'sineCurve': QEasingCurve.Type.SineCurve,
            'cosineCurve': QEasingCurve.Type.CosineCurve,
        }
        t = easing_map.get(ease_data, QEasingCurve.Type.OutCubic)
        return QEasingCurve(t)
    # default
    return QEasingCurve(QEasingCurve.Type.OutCubic)




import uuid
from typing import Optional, Union, Dict, List, Callable
from PySide6.QtWidgets import QHBoxLayout, QWidget
from PySide6.QtCore import Qt, QSize
from qtmui.hooks import State
from qtmui.material.styles import useTheme
from ..utils.validate_params import _validate_param
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style

class ListItemAvatar(QWidget):
    """
    A component that renders an avatar within a ListItem, styled like Material-UI ListItemAvatar.

    The `ListItemAvatar` component is used to display an avatar (typically an Avatar component)
    within a ListItem, with customizable styles and alignment.

    Parameters
    ----------
    children : State, QWidget, List[QWidget], or None, optional
        The content of the component, typically an Avatar. Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    sx : State, list, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QWidget` class,
        supporting props of the native component (e.g., parent, style, className).

    Notes
    -----
    - Props of the native component are supported via `**kwargs` (e.g., `parent`, `style`, `className`).
    - The `children` prop is typically an Avatar component but can be any QWidget or list of QWidgets.
    - The component uses a QHBoxLayout to align the avatar to the left, consistent with Material-UI.

    Demos:
    - ListItemAvatar: https://qtmui.com/material-ui/qtmui-listitemavatar/

    API Reference:
    - ListItemAvatar API: https://qtmui.com/material-ui/api/list-item-avatar/
    """

    def __init__(
        self,
        children: Optional[Union[State, QWidget, List[QWidget]]] = None,
        classes: Optional[Union[State, Dict]] = None,
        sx: Optional[Union[State, List, Dict, Callable, str]] = None,
        **kwargs
    ):
        super().__init__(**kwargs)
        self.setObjectName(f"ListItemAvatar-{str(uuid.uuid4())}")

        self.theme = useTheme()
        self._widget_references = []

        # Set properties with validation
        self._set_children(children)
        self._set_classes(classes)
        self._set_sx(sx)

        self._init_ui()


    # Setter and Getter methods
    @_validate_param(file_path="qtmui.material.listitemavatar", param_name="children", supported_signatures=Union[State, QWidget, List, type(None)])
    def _set_children(self, value):
        """Assign value to children and store references."""
        self._children = value

    def _get_children(self):
        """Get the children value."""
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.listitemavatar", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        """Assign value to classes."""
        self._classes = value

    def _get_classes(self):
        """Get the classes value."""
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.listitemavatar", param_name="sx", supported_signatures=Union[State, List, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        """Assign value to sx."""
        self._sx = value

    def _get_sx(self):
        """Get the sx value."""
        return self._sx.value if isinstance(self._sx, State) else self._sx

    def _init_ui(self):
        # Layout cơ bản cho icon
        self._layout = QHBoxLayout(self)
        self._layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(self._layout)
        # self.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        self.setFixedSize(QSize(32, 32))
        # self.setFixedWidth(32)
        self.setAttribute(Qt.WA_TransparentForMouseEvents) # Điều này cho phép các sự kiện chuột (bao gồm hover) có thể đi qua và được lắng nghe bởi QPushButton.

        # Thêm các children (nếu có)
        if self._children:
            if not isinstance(self._children, list):
                raise TypeError("children must be type (list)")
            for child in self._children:
                self.layout().addWidget(child)


        # Áp dụng các styles thông qua sx
        if self._sx:
            self._apply_sx(self._sx)

        # Áp dụng các class styles thông qua classes
        if self._classes:
            self._apply_classes(self._classes)

    def _apply_sx(self, sx):
        """Áp dụng các overrides styles từ sx"""
        if isinstance(sx, dict):
            for key, value in sx.items():
                self.setStyleSheet(f"{key}: {value};")
        elif callable(sx):
            sx(self)

    def _apply_classes(self, classes):
        """Áp dụng class CSS từ classes"""
        if isinstance(classes, dict):
            styles = "; ".join([f"{k}: {v}" for k, v in classes.items()])
            self.setStyleSheet(styles)



# from .qtmui_window import QtMuiWindow, MSFluentWindow, SplitFluentWindow, FluentTitleBar, MSFluentTitleBar, SplitTitleBar, FluentBackgroundTheme
# from .splash_screen import SplashScreen
from .qtmui_window import QtMuiWindow


# Copyright (C) 2022 The Pyact Company Ltd.
# SPDX-License-Identifier: LicenseRef-Pyact-Commercial OR BSD-3-Clause
from __future__ import annotations

"""..site_packages.qtcompat port of the areachart example from qtmui v1.0"""

from typing import Optional, Union
import sys
from PySide6.QtCore import QUrl
from PySide6.QtWebEngineWidgets import QWebEngineView

from qtmui.material.styles import useTheme



class MapChangeTheme(QWebEngineView):
    def __init__(
                self,
                initialViewState: dict = None, 
                *args,
                **kwargs
                ):
        super().__init__(*args, **kwargs)

        self._autorun_set_theme()

 
    def _init_map(self):
        self.theme = useTheme()
        self.setUrl(QUrl("https://minimals.cc/components/extra/map"))


    def _autorun_set_theme(self):
        @store.autorun(lambda state: state.theme.palette)
        def _(palette):
            try:
                self._init_map()
            except Exception as e:
                pass
        if not hasattr(self, "theme"):
            self._init_map()


import asyncio
from typing import Callable, Optional, Union
from PySide6.QtWidgets import QHBoxLayout
from PySide6.QtCore import Qt, QTimer

from qtmui.hooks import State
from .button import Button
# from .arc_widget import ArcWidget
from .loading_icon import LoadingIcon
from ..py_svg_widget import PySvgWidget

from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from qtmui.material.styles import useTheme
from qtmui.i18n.use_translation import translate, i18n

class LoadingButton(Button):
    """
    LoadingButton
    Base Button

    Args:
        loading?: True | False
        loadingPosition?: "start" | "center" | "end"
        loadingIndicator?: "Loading..." | any str

    Returns:
        new instance of LoadingButton
    """
    def __init__(self,
                type: str = None,
                loading: bool = False,
                loadingPosition: str = "start",
                loadingIndicator: Optional[Union[str, State, Callable]] = None,
                color: str = None,
                 *args, **kwargs):
        if loading:
            super().__init__(disabled=True, *args, **kwargs)
        else:
            super().__init__(*args, **kwargs)

        self._type = type
        self._loading = loading
        self._loadingPosition = loadingPosition
        self._loadingIndicator = loadingIndicator

        self._init_ui()

    def _init_ui(self):

        if self._loading:
            # Tạo layout để quản lý icon và text
            self.hbox = QHBoxLayout(self)
            self.hbox.setContentsMargins(0, 0, 0, 0)
            self.hbox.setSpacing(5)  # Khoảng cách giữa icon và text
            self.setLayout(self.hbox)

            if self._loadingIndicator is not None:
                self.setText(self._loadingIndicator)
            else:
                # self.setLayout(QHBoxLayout())
                # self.layout().setContentsMargins(0,0,0,0)
                self.loadingSvgWidget = PySvgWidget(key="eos-icons:loading")

                # if self._loadingPosition == "start":
                #     self.layout().addWidget(self.loadingSvgWidget, alignment=Qt.AlignLeft)
                # else:
                #     self.layout().addWidget(self.loadingSvgWidget, alignment=Qt.AlignRight)
                #     # self.setLayoutDirection(Qt.RightToLeft)

                if self._loadingPosition == "start":
                    self.hbox.addWidget(self.loadingSvgWidget, alignment=Qt.AlignLeft)
                    self.hbox.addStretch()
                    self.hbox.addWidget(self)
                elif self._loadingPosition == "end":
                    self.hbox.addWidget(self)
                    self.hbox.addStretch()
                    self.hbox.addWidget(self.loadingSvgWidget, alignment=Qt.AlignRight)

        self.theme = useTheme()
        
        self.theme.state.valueChanged.connect(self._onThemeChanged)
        # QTimer.singleShot(0, self._scheduleSetStyleSheet)
        self._set_stylesheet()
        
        
        self.destroyed.connect(lambda obj: self._onDestroy())

    def _onDestroy(self, obj=None):
        if hasattr(self, "_setupStyleSheet") and self._setupStyleSheet and not self._setupStyleSheet.done():
            self._setupStyleSheet.cancel()

    def _onThemeChanged(self):
        if not self.isVisible():
            return
        QTimer.singleShot(0, self._scheduleSetStyleSheet)

    def _scheduleSetStyleSheet(self):
        self._setupStyleSheet = asyncio.ensure_future(self._lazy_set_stylesheet())

    async def _lazy_set_stylesheet(self):
        self.__set_loading_button_svg_stylesheet()

    def __set_loading_button_svg_stylesheet(self, _theme=None):
        self.theme = useTheme()
        component_styles = self.theme.components

        ownerState = {
            "size": self._size
        }
        PyLoadingButton_root = component_styles["PyLoadingButton"].get("styles")["root"](ownerState)
        PyLoadingButton_root_prop_softVariant_loadingIndicatorStart_left = PyLoadingButton_root["props"]["softVariant"]["loadingIndicatorStart"]["left"]
        PyLoadingButton_root_prop_softVariant_loadingIndicatorEnd_right = PyLoadingButton_root["props"]["softVariant"]["loadingIndicatorEnd"]["right"]
        PyLoadingButton_root_prop_smallSize_loadingIndicatorStart_left = PyLoadingButton_root["props"]["smallSize"]["loadingIndicatorStart"]["left"]
        PyLoadingButton_root_prop_smallSize_loadingIndicatorEnd_right = PyLoadingButton_root["props"]["smallSize"]["loadingIndicatorEnd"]["right"]



import re
import sys
from typing import Any, Dict, Optional, Callable, Tuple

from PySide6 import QtAsyncio

from PySide6.QtWidgets import QFrame, QVBoxLayout
from PySide6.QtCore import QSize, Qt, QTimer, QRectF, QPointF
from PySide6.QtGui import (
    QPainter, QColor, QPen, QPixmap,
    QBrush, QLinearGradient, QPainterPath
)

from qtmui.material.qtmui_app import QtMuiApp
from qtmui.material.window import QtMuiWindow
from qtmui.material.button import Button

from flow_layout import FlowLayout


# =============================================================================
# Helper: StyleDecorator (border + radius + background + cache pixmap)
# =============================================================================

class StyleDecorator:
    """
    Helper xử lý style dạng sx cho QWidget:
      - Parse sx 1 lần (tránh parse/alloc trong paintEvent)
      - Render ra pixmap cache theo (size, dpr, styleKey)
      - Paint: chỉ drawPixmap (nhẹ)

    Hỗ trợ:
      - border (4 cạnh khác nhau hoặc đồng nhất)
      - border-radius (uniform hoặc theo từng góc)
      - background: color / linear-gradient(...)
    """

    def __init__(self, owner: QFrame, sx: Optional[Dict[str, Any]] = None):
        self._owner = owner
        self._sx: Dict[str, Any] = sx or {}

        self._border_spec: Dict[str, Dict[str, Any]] = {}
        self._radius_spec: Dict[str, float] = {}
        self._bg_spec: Dict[str, Any] = {}

        self._cache_size: Optional[Tuple[int, int]] = None
        self._cache_dpr: Optional[float] = None
        self._cache_key: Optional[str] = None
        self._pixmap: Optional[QPixmap] = None

        self.set_sx(self._sx)

    # ------------------------- Public API -------------------------

    def set_sx(self, sx: Dict[str, Any]):
        self._sx = sx or {}
        self._border_spec = self._parse_border_sx(self._sx)
        self._radius_spec = self._parse_radius_sx(self._sx)
        self._bg_spec = self._parse_background_sx(self._sx)
        self.invalidate_cache()

    def invalidate_cache(self):
        self._cache_size = None
        self._cache_dpr = None
        self._cache_key = None
        self._pixmap = None

    def paint(self, painter: QPainter, rect_size: QSize, dpr: float):
        pm = self._get_pixmap(rect_size, dpr)
        if pm is None or pm.isNull():
            return
        painter.drawPixmap(0, 0, pm)

    # ------------------------- Enum helpers (PySide6-safe) -------------------------

    def _enum_to_int(self, v) -> int:
        try:
            if isinstance(v, int):
                return v
            if hasattr(v, "value"):
                return int(v.value)
            return int(v)
        except Exception:
            return 0

    # ------------------------- Parsing utils -------------------------

    def _parse_px(self, s: str) -> float:
        if not isinstance(s, str):
            return 0.0
        m = re.search(r"(-?\d+(?:\.\d+)?)\s*px", s.strip(), re.I)
        if not m:
            m = re.search(r"(-?\d+(?:\.\d+)?)", s.strip())
        if not m:
            return 0.0
        try:
            return max(0.0, float(m.group(1)))
        except Exception:
            return 0.0

    def _parse_color(self, v: Any) -> QColor:
        if v is None:
            return QColor(0, 0, 0, 0)
        if isinstance(v, QColor):
            return v
        if isinstance(v, (tuple, list)) and len(v) in (3, 4):
            r, g, b = int(v[0]), int(v[1]), int(v[2])
            a = int(v[3]) if len(v) == 4 else 255
            return QColor(r, g, b, max(0, min(255, a)))

        if not isinstance(v, str):
            return QColor(0, 0, 0, 0)

        s = v.strip()

        m = re.search(
            r"rgba?\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)(?:\s*,\s*([0-9.]+)\s*)?\)",
            s, re.I
        )
        if m:
            r = int(m.group(1))
            g = int(m.group(2))
            b = int(m.group(3))
            a_raw = m.group(4)
            if a_raw is None:
                a = 255
            else:
                a_f = float(a_raw)
                a = int(round(a_f * 255)) if a_f <= 1.0 else int(round(a_f))
                a = max(0, min(255, a))
            return QColor(r, g, b, a)

        if s.startswith("#"):
            c = QColor(s)
            if c.isValid():
                return c

        c = QColor(s)
        if c.isValid():
            return c

        return QColor(0, 0, 0, 0)

    def _parse_pen_style_int(self, s: str) -> int:
        ss = (s or "").lower()
        if "dashed" in ss:
            return self._enum_to_int(Qt.PenStyle.DashLine)
        if "dotted" in ss:
            return self._enum_to_int(Qt.PenStyle.DotLine)
        return self._enum_to_int(Qt.PenStyle.SolidLine)

    # ------------------------- Border parsing -------------------------

    def _parse_border_value(self, v: Any) -> Dict[str, Any]:
        """
        Output: {"w": int, "color": QColor, "style": int(enum_value)}
        """
        solid_i = self._enum_to_int(Qt.PenStyle.SolidLine)

        if v is None:
            return {"w": 0, "color": QColor(0, 0, 0, 0), "style": solid_i}

        if isinstance(v, (int, float)):
            w = max(0, int(round(v)))
            return {"w": w, "color": QColor(0, 0, 0, 255), "style": solid_i}

        if not isinstance(v, str):
            return {"w": 0, "color": QColor(0, 0, 0, 0), "style": solid_i}

        s = v.strip()
        w = int(round(self._parse_px(s)))
        style_i = self._parse_pen_style_int(s)

        m_rgba = re.search(r"rgba?\([^)]+\)", s, re.I)
        if m_rgba:
            color = self._parse_color(m_rgba.group(0))
        else:
            m_hex = re.search(r"(#(?:[0-9a-fA-F]{6}|[0-9a-fA-F]{8}))", s)
            if m_hex:
                color = self._parse_color(m_hex.group(1))
            else:
                color = QColor(0, 0, 0, 255)

        return {"w": w, "color": color, "style": style_i}

    def _parse_border_sx(self, sx: Dict[str, Any]) -> Dict[str, Dict[str, Any]]:
        base = self._parse_border_value(sx.get("border") or sx.get("borderWidth") or sx.get("border-width"))

        def pick(*keys):
            for k in keys:
                if k in sx and sx[k] is not None:
                    return sx[k]
            return None

        left_v = pick("borderLeft", "border-left", "borderLeftWidth", "border-left-width")
        right_v = pick("borderRight", "border-right", "borderRightWidth", "border-right-width")
        top_v = pick("borderTop", "border-top", "borderTopWidth", "border-top-width")
        bottom_v = pick("borderBottom", "border-bottom", "borderBottomWidth", "border-bottom-width")

        left = self._parse_border_value(left_v) if left_v is not None else base
        right = self._parse_border_value(right_v) if right_v is not None else base
        top = self._parse_border_value(top_v) if top_v is not None else base
        bottom = self._parse_border_value(bottom_v) if bottom_v is not None else base

        return {"left": left, "right": right, "top": top, "bottom": bottom}

    # ------------------------- Radius parsing -------------------------

    def _parse_radius_sx(self, sx: Dict[str, Any]) -> Dict[str, float]:
        base = sx.get("borderRadius", sx.get("border-radius", 0))
        base_r = float(base) if isinstance(base, (int, float)) else float(self._parse_px(str(base)))
        base_r = max(0.0, base_r)

        def pick_radius(*keys) -> float:
            for k in keys:
                if k in sx and sx[k] is not None:
                    v = sx[k]
                    if isinstance(v, (int, float)):
                        return max(0.0, float(v))
                    return max(0.0, float(self._parse_px(str(v))))
            return base_r

        tl = pick_radius("borderTopLeftRadius", "border-top-left-radius")
        tr = pick_radius("borderTopRightRadius", "border-top-right-radius")
        br = pick_radius("borderBottomRightRadius", "border-bottom-right-radius")
        bl = pick_radius("borderBottomLeftRadius", "border-bottom-left-radius")

        return {"tl": tl, "tr": tr, "br": br, "bl": bl}

    # ------------------------- Background parsing -------------------------

    def _parse_linear_gradient(self, s: str) -> Optional[Dict[str, Any]]:
        s = s.strip()
        if not s.lower().startswith("linear-gradient(") or not s.endswith(")"):
            return None

        inner = s[len("linear-gradient("):-1].strip()
        parts = [p.strip() for p in inner.split(",") if p.strip()]
        if len(parts) < 2:
            return None

        direction = "to bottom"
        colors = parts

        first = parts[0].lower()
        if first.startswith("to ") or first.endswith("deg"):
            direction = parts[0]
            colors = parts[1:]

        if len(colors) < 2:
            return None

        c1 = self._parse_color(colors[0])
        c2 = self._parse_color(colors[1])

        return {"type": "linear-gradient", "dir": direction, "c1": c1, "c2": c2}

    def _parse_background_sx(self, sx: Dict[str, Any]) -> Dict[str, Any]:
        v = sx.get("background", None)
        if v is None:
            v = sx.get("backgroundColor", sx.get("background-color", None))

        if v is None:
            return {"type": "none"}

        if isinstance(v, str):
            grad = self._parse_linear_gradient(v)
            if grad:
                return grad

        return {"type": "color", "color": self._parse_color(v)}

    # ------------------------- Cache key -------------------------

    def _style_key(self) -> str:
        b = self._border_spec
        r = self._radius_spec
        bg = self._bg_spec

        def border_one(side: str) -> str:
            s = b.get(side, {})
            c: QColor = s.get("color", QColor(0, 0, 0, 0))
            style_i = self._enum_to_int(s.get("style", self._enum_to_int(Qt.PenStyle.SolidLine)))
            return f"{side}:{int(s.get('w', 0) or 0)}:{style_i}:{c.rgba()}"

        b_key = "|".join([border_one("left"), border_one("top"), border_one("right"), border_one("bottom")])
        r_key = f"r:{r.get('tl',0):.2f},{r.get('tr',0):.2f},{r.get('br',0):.2f},{r.get('bl',0):.2f}"

        if bg.get("type") == "none":
            bg_key = "bg:none"
        elif bg.get("type") == "color":
            c: QColor = bg.get("color", QColor(0, 0, 0, 0))
            bg_key = f"bg:color:{c.rgba()}"
        else:
            c1: QColor = bg.get("c1", QColor(0, 0, 0, 0))
            c2: QColor = bg.get("c2", QColor(0, 0, 0, 0))
            bg_key = f"bg:grad:{bg.get('dir','')}:{c1.rgba()}:{c2.rgba()}"

        return f"{b_key}||{r_key}||{bg_key}"

    # ------------------------- Pixmap render -------------------------

    def _get_pixmap(self, size: QSize, dpr: float) -> Optional[QPixmap]:
        w = max(0, int(size.width()))
        h = max(0, int(size.height()))
        if w <= 0 or h <= 0:
            return None

        key = self._style_key()

        if (
            self._pixmap is not None
            and self._cache_size == (w, h)
            and self._cache_dpr == dpr
            and self._cache_key == key
        ):
            return self._pixmap

        pm = QPixmap(int(w * dpr), int(h * dpr))
        pm.setDevicePixelRatio(dpr)
        pm.fill(Qt.GlobalColor.transparent)

        p = QPainter(pm)
        p.setRenderHint(QPainter.RenderHint.Antialiasing, True)

        self._render(p, float(w), float(h))

        p.end()

        self._pixmap = pm
        self._cache_size = (w, h)
        self._cache_dpr = dpr
        self._cache_key = key
        return pm

    def _render(self, p: QPainter, w: float, h: float):
        self._render_background(p, w, h)
        self._render_border(p, w, h)

    def _rounded_path(self, rect: QRectF, r: Dict[str, float]) -> QPainterPath:
        tl = float(r.get("tl", 0.0))
        tr = float(r.get("tr", 0.0))
        br = float(r.get("br", 0.0))
        bl = float(r.get("bl", 0.0))

        max_rx = rect.width() / 2.0
        max_ry = rect.height() / 2.0
        tl = min(tl, max_rx, max_ry)
        tr = min(tr, max_rx, max_ry)
        br = min(br, max_rx, max_ry)
        bl = min(bl, max_rx, max_ry)

        x, y, rw, rh = rect.x(), rect.y(), rect.width(), rect.height()
        path = QPainterPath()

        path.moveTo(x + tl, y)
        path.lineTo(x + rw - tr, y)
        if tr > 0:
            path.quadTo(x + rw, y, x + rw, y + tr)
        else:
            path.lineTo(x + rw, y)

        path.lineTo(x + rw, y + rh - br)
        if br > 0:
            path.quadTo(x + rw, y + rh, x + rw - br, y + rh)
        else:
            path.lineTo(x + rw, y + rh)

        path.lineTo(x + bl, y + rh)
        if bl > 0:
            path.quadTo(x, y + rh, x, y + rh - bl)
        else:
            path.lineTo(x, y + rh)

        path.lineTo(x, y + tl)
        if tl > 0:
            path.quadTo(x, y, x + tl, y)
        else:
            path.lineTo(x, y)

        path.closeSubpath()
        return path

    def _render_background(self, p: QPainter, w: float, h: float):
        bg = self._bg_spec
        if bg.get("type") == "none":
            return

        rect = QRectF(0.0, 0.0, w, h)
        r = self._radius_spec
        has_radius = any((r.get("tl", 0), r.get("tr", 0), r.get("br", 0), r.get("bl", 0)))

        if has_radius:
            path = self._rounded_path(rect, r)
            p.save()
            p.setClipPath(path)

        if bg.get("type") == "color":
            c: QColor = bg.get("color", QColor(0, 0, 0, 0))
            p.fillRect(rect, QBrush(c))
        elif bg.get("type") == "linear-gradient":
            grad = self._build_linear_gradient(w, h, bg.get("dir", "to bottom"), bg.get("c1"), bg.get("c2"))
            p.fillRect(rect, QBrush(grad))

        if has_radius:
            p.restore()

    def _build_linear_gradient(self, w: float, h: float, direction: str, c1: QColor, c2: QColor) -> QLinearGradient:
        direction = (direction or "").strip().lower()
        c1 = c1 if isinstance(c1, QColor) else QColor(0, 0, 0, 0)
        c2 = c2 if isinstance(c2, QColor) else QColor(0, 0, 0, 0)

        x1, y1, x2, y2 = 0.0, 0.0, 0.0, h

        if direction.startswith("to "):
            if "right" in direction:
                x1, y1, x2, y2 = 0.0, 0.0, w, 0.0
            elif "left" in direction:
                x1, y1, x2, y2 = w, 0.0, 0.0, 0.0
            elif "top" in direction:
                x1, y1, x2, y2 = 0.0, h, 0.0, 0.0
            else:
                x1, y1, x2, y2 = 0.0, 0.0, 0.0, h
        elif direction.endswith("deg"):
            try:
                import math
                deg = float(direction.replace("deg", "").strip())
                rad = math.radians(deg)
                vx = math.sin(rad)
                vy = -math.cos(rad)
                cx, cy = w / 2.0, h / 2.0
                half = max(w, h) / 2.0
                x1, y1 = cx - vx * half, cy - vy * half
                x2, y2 = cx + vx * half, cy + vy * half
            except Exception:
                x1, y1, x2, y2 = 0.0, 0.0, 0.0, h

        g = QLinearGradient(QPointF(x1, y1), QPointF(x2, y2))
        g.setColorAt(0.0, c1)
        g.setColorAt(1.0, c2)
        return g

    def _border_is_uniform(self) -> bool:
        s = self._border_spec
        a = s["left"]
        for k in ("top", "right", "bottom"):
            b = s[k]
            if int(a.get("w", 0) or 0) != int(b.get("w", 0) or 0):
                return False
            if int(a.get("style", 0) or 0) != int(b.get("style", 0) or 0):
                return False
            ca: QColor = a.get("color", QColor(0, 0, 0, 0))
            cb: QColor = b.get("color", QColor(0, 0, 0, 0))
            if ca.rgba() != cb.rgba():
                return False
        return int(a.get("w", 0) or 0) > 0

    def _render_border(self, p: QPainter, w: float, h: float):
        s = self._border_spec
        lw = int(s["left"].get("w", 0) or 0)
        rw = int(s["right"].get("w", 0) or 0)
        tw = int(s["top"].get("w", 0) or 0)
        bw = int(s["bottom"].get("w", 0) or 0)

        if (lw + rw + tw + bw) <= 0:
            return

        if self._border_is_uniform():
            self._render_uniform_border(p, w, h)
            return

        x_left = lw / 2.0 if lw > 0 else 0.0
        x_right = (w - rw / 2.0) if rw > 0 else w
        y_top = tw / 2.0 if tw > 0 else 0.0
        y_bottom = (h - bw / 2.0) if bw > 0 else h

        def set_pen(side_spec: Dict[str, Any]):
            bw_ = int(side_spec.get("w", 0) or 0)
            col: QColor = side_spec.get("color", QColor(0, 0, 0, 255))
            style_i = int(side_spec.get("style", self._enum_to_int(Qt.PenStyle.SolidLine)) or 0)
            pen = QPen(col, bw_, Qt.PenStyle(style_i))
            pen.setCapStyle(Qt.PenCapStyle.SquareCap)
            p.setPen(pen)

        if tw > 0:
            set_pen(s["top"])
            p.drawLine(QPointF(x_left, y_top), QPointF(x_right, y_top))
        if bw > 0:
            set_pen(s["bottom"])
            p.drawLine(QPointF(x_left, y_bottom), QPointF(x_right, y_bottom))
        if lw > 0:
            set_pen(s["left"])
            p.drawLine(QPointF(x_left, y_top), QPointF(x_left, y_bottom))
        if rw > 0:
            set_pen(s["right"])
            p.drawLine(QPointF(x_right, y_top), QPointF(x_right, y_bottom))

    def _render_uniform_border(self, p: QPainter, w: float, h: float):
        s = self._border_spec["left"]
        bw = int(s.get("w", 0) or 0)
        if bw <= 0:
            return

        col: QColor = s.get("color", QColor(0, 0, 0, 255))
        style_i = int(s.get("style", self._enum_to_int(Qt.PenStyle.SolidLine)) or 0)
        pen = QPen(col, bw, Qt.PenStyle(style_i))
        pen.setCapStyle(Qt.PenCapStyle.SquareCap)
        p.setPen(pen)
        p.setBrush(Qt.BrushStyle.NoBrush)

        inset = bw / 2.0
        rect = QRectF(inset, inset, max(0.0, w - 2 * inset), max(0.0, h - 2 * inset))

        r = self._radius_spec
        has_radius = any((r.get("tl", 0), r.get("tr", 0), r.get("br", 0), r.get("bl", 0)))
        if not has_radius:
            p.drawRect(rect)
            return

        rr = {
            "tl": max(0.0, r.get("tl", 0.0) - inset),
            "tr": max(0.0, r.get("tr", 0.0) - inset),
            "br": max(0.0, r.get("br", 0.0) - inset),
            "bl": max(0.0, r.get("bl", 0.0) - inset),
        }
        path = self._rounded_path(rect, rr)
        p.drawPath(path)


# =============================================================================
# Demo widgets
# =============================================================================

class Item(QFrame):
    def __init__(self, width=100, height=20, color="red"):
        super().__init__()
        self.setObjectName(str(id(self)))
        self.setFixedSize(QSize(width, height))
        self.setStyleSheet(f"#{self.objectName()} {{background: rgba(0,0,0,50);}}")


class Box(QFrame):
    def __init__(self, children=None):
        super().__init__()
        self.setObjectName(str(id(self)))
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)
        for widget in children:
            self.layout().addWidget(widget)


class FlexBoxWrap(QFrame):
    def __init__(
        self,
        name="FlexBoxWrap",
        children=None,
        flexWrap=True,
        align="left",
        color="blue",
        onLayout: Optional[Callable[[Dict[str, Any]], None]] = None,
        sx: dict = None
    ):
        super().__init__()
        self.setObjectName(str(id(self)))

        self._sx: Dict[str, Any] = sx or {"border": "1px solid rgba(0,0,0,100)"}
        self._decor = StyleDecorator(self, self._sx)

        # Không dùng stylesheet border để tránh Qt shrink contentsRect
        self.setLayout(
            FlowLayout(
                name=name,
                parent=self,
                children=children,
                alignItems="center",
                justifyContent="flex-start",
                sx={},
            )
        )

        self._repaint_scheduled = False

    def setSx(self, sx: Dict[str, Any]):
        self._sx = sx or {}
        self._decor.set_sx(self._sx)
        self._schedule_repaint()

    def _schedule_repaint(self):
        if self._repaint_scheduled:
            return
        self._repaint_scheduled = True

        def _go():
            self._repaint_scheduled = False
            self.update()

        QTimer.singleShot(0, _go)

    def resizeEvent(self, event):
        super().resizeEvent(event)
        self._decor.invalidate_cache()
        self._schedule_repaint()

    def paintEvent(self, event):
        super().paintEvent(event)
        p = QPainter(self)
        self._decor.paint(p, self.size(), self.devicePixelRatioF())
        p.end()


class MainWindow(QtMuiWindow):
    def __init__(self):
        super().__init__()
        self.resize(300, 300)

        def _setMainWindowSize():
            self.resize(230, 300)

        self.setCentralWidget(
            Box(
                children=[
                    Button(text="230px", variant="contained", onClick=lambda: _setMainWindowSize()),
                    FlexBoxWrap(
                        name="FBW",
                        sx={
                            "border": "1px solid rgba(0,0,0,100)",
                            "borderRadius": "10px",
                            "background": "linear-gradient(to right, rgba(0,0,0,30), rgba(0,0,0,5))",
                        },
                        children=[
                            FlexBoxWrap(
                                name="FBW-1",
                                sx={
                                    "border": "1px solid rgba(0,0,0,100)",
                                    "borderRadius": "10px",
                                    "backgroundColor": "rgba(0,0,0,10)",
                                },
                                children=[Item(), Item()],
                            ),
                            Item(),
                        ],
                    ),
                ]
            )
        )


if __name__ == "__main__":
    app = QtMuiApp(sys.argv)
    window = MainWindow()
    window.show()
    sys.exit(app.exec())



import uuid
from PySide6.QtWidgets import QVBoxLayout, QFrame
from typing import Union, List, Callable

from qtmui.material.styles.create_theme.theme_reducer import ThemeState
from qtmui.material.styles.create_theme.create_palette import PaletteColor
from ..avatar import Avatar

from qtmui.material.styles import useTheme
from qtmui.i18n.use_translation import translate, i18n
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from ..widget_base import PyWidgetBase

class TimelineDot(QFrame, PyWidgetBase):
    def __init__(self,
                 children=None,  # Nội dung của component (node)
                 classes: dict = None,  # Ghi đè hoặc mở rộng các styles áp dụng cho component (dict)
                 color: Union[str, None] = 'default',  # Màu sắc của dot, mặc định là 'grey'
                 sx: Union[List[Union[Callable, dict, bool]], Callable, dict] = None,  # Thuộc tính hệ thống, thêm CSS bổ sung (array | func | object)
                 variant: str = 'filled'  # Kiểu của dot, có thể là 'filled' hoặc 'outlined', mặc định là 'filled'
                 ):
        super().__init__()
        self.setObjectName(str(uuid.uuid4()))
        # self.setStyleSheet(f"#{self.objectName()} {{background-color: yellow;}}")

        self._children = children
        self._classes = classes if classes is not None else {}
        self._color = color
        self._sx = sx if sx is not None else []
        self._variant = variant



        i18n.langChanged.connect(self.retranslateUi)
        self.retranslateUi()
        
        self.slot_set_stylesheet()
        self.theme = useTheme()
        self.theme.state.valueChanged.connect(self.slot_set_stylesheet)
        self.destroyed.connect(self._on_destroyed)

        # Khởi tạo giao diện của dot
        self._init_ui()

    def slot_set_stylesheet(self, value=None):
        self._set_stylesheet()


    def _init_ui(self):
        """Khởi tạo giao diện của TimelineItem."""
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)


        if not self._children:
            self.layout().addWidget(Avatar(size=13, text=" ", color=self._dot_color))
        else:
            self._add_children()

    def retranslateUi(self):
       pass

    def _set_stylesheet(self, component_styled=None):
        self.theme = useTheme()

        if not component_styled:
            component_styled = self.theme.components

        if self._color == None:
            self._dot_color = self.theme.palette.grey._500
        elif self._color.find("#") == -1 and self._color in ['primary', 'secondary', 'info', 'success', 'warning', 'error']:
            self._dot_color = component_styled["PyTimeLine"]["styles"]["dot"][self._color]["color"]
        else:
            self._dot_color = self._color


    def _add_children(self):
        if self._children:
            if not isinstance(self._children, list):
                raise TypeError("children must be type (list)")
            for child in self._children:
                if child:
                    self.layout().addWidget(child)


from typing import Dict
from qtmui.material.styles.create_theme.theme_reducer import ThemeState

def slider(theme: ThemeState) -> Dict:
    light_mode = theme.palette.mode == 'light'

    return {
        "MuiSlider": {
            "styleOverrides": {
                "root": {
                    f"&.{sliderClasses.disabled}": {
                        "color": theme.palette.action.disabled,
                    },
                },
                "rail": {
                    "opacity": 0.32,
                },
                "markLabel": {
                    "fontSize": 13,
                    "color": theme.palette.text.disabled,
                },
                "valueLabel": {
                    "borderRadius": 8,
                    "backgroundColor": theme.palette.grey[800 if light_mode else 700],
                },
            },
        },
    }
